import sys
from argparse import ArgumentParser


class Parser(object):
    """Argument parser that can handle arguments with our special
    placeholder.

    """

    def __init__(self):
        self._parser = ArgumentParser(prog='lmgbuild', add_help=False)
        self._add_arguments()

    def _add_arguments(self):
        """Adds arguments to parser."""
        self._parser.add_argument(
            '-v', '--version',
            action='store_true',
            help="show program's version number and exit")
        self._parser.add_argument(
            '-h', '--help',
            action='store_true',
            help='show this help message and exit')
        self._parser.add_argument(
            "-s","--sync",
            action='store_true',
            help='sync repository')
        self._add_conflicting_arguments()
        self._parser.add_argument(
            '-d', '--debug',
            action='store_true',
            help='enable debug output')
#        self._parser.add_argument(
#            'command',
#            nargs='*',
#            help='command that should be fixed')
        subparsers = self._parser.add_subparsers(help='sub command help')
        parser_subr = subparsers.add_parser('runtime',aliases=['rt'],help='runtime command....')
        parser_subr.add_argument('-s','--showruntime',action='store_true',help='show baseruntime ...')
        parser_subr.add_argument('-d','--deleteruntime',dest='druntime',help='delete baseruntime ...')
        parser_subr.add_argument('-i','--loadruntime',dest='lruntime',help='load baseruntime ...')
        parser_subw = subparsers.add_parser('workspace',aliases=['ws'],help='workspace command....')
        parser_subw.add_argument('-s','--showorkspace',action='store_true',help='show baseruntime ...')
#        parser_sub.add_argument('-w',action='store_true',help='workspace')
#        parser_sub.set_defaults(func=print_help)

    def _add_conflicting_arguments(self):
        """It's too dangerous to use `-y` and `-r` together."""
        group = self._parser.add_mutually_exclusive_group()
#        group.add_argument(
#            '-i','--import',
#            dest='runtime',
#            help='(import baseruntime for lmgbuild')
        group.add_argument(
            "-w","--workdir",
            dest='workdir',
            help='set workdir for lmgbuild')
        group.add_argument(
            "-c","--config",
            dest='config',
            help='batch build config file')
    def parse(self, argv):
        return self._parser.parse_args(argv[1:])

    def print_usage(self):
        self._parser.print_usage(sys.stderr)

    def print_help(self):
        self._parser.print_help(sys.stderr)
