import os
import sys
from warnings import warn
from six import text_type
#from . import const
import lmgbuild.const as const
from pathlib import Path
import importlib


class Settings(dict):
    def __getattr__(self, item):
        return self.get(item)

    def __setattr__(self, key, value):
        self[key] = value

    def init(self, args=None):
        """Fills `settings` with values from `settings.py` and env."""

        from lmgbuild.logs import exception,debug
        debug("conf init ....")

        self._setup_user_dir()
        self._init_settings_file()

        try:
            self.update(self._settings_from_file())
        except Exception:
            exception("Can't load settings from file", sys.exc_info())

        try:
            self.update(self._settings_from_env())
        except Exception:
            exception("Can't load settings from env", sys.exc_info())

        self.update(self._settings_from_args(args))
        debug("settings: %s" % self)

    def _init_settings_file(self):
        settings_path = self.user_dir.joinpath('settings.py')
        if not settings_path.is_file():
            with settings_path.open(mode='w') as settings_file:
                settings_file.write(const.SETTINGS_HEADER)
                for setting in const.DEFAULT_SETTINGS.items():
                    settings_file.write(u'# {} = {}\n'.format(*setting))

    def _get_user_dir_path(self):
        """Returns Path object representing the user config resource"""
        xdg_config_home = os.environ.get('XDG_CONFIG_HOME', '~/.config')
        user_dir = Path(xdg_config_home, 'lmgbuild').expanduser()
        legacy_user_dir = Path('~', '.lmgbuild').expanduser()

        # For backward compatibility use legacy '~/.thefuck' if it exists:
        if legacy_user_dir.is_dir():
            warn(u'Config path {} is deprecated. Please move to {}'.format(
                legacy_user_dir, user_dir))
            return legacy_user_dir
        else:
            return user_dir

    def _setup_user_dir(self):
        """Returns user config dir, create it when it doesn't exist."""
        user_dir = self._get_user_dir_path()
        self.user_dir = user_dir

    def _settings_from_file(self):
        """Loads see.tings from file."""
        spec = importlib.util.spec_from_file_location('settings',text_type(self.user_dir.joinpath('settings.py')))
        settings = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(settings)
        return {key: getattr(settings, key)
                for key in const.DEFAULT_SETTINGS.keys()
                if hasattr(settings, key)}

    def _val_from_env(self, env, attr):
        """Transforms env-strings to python."""
        val = os.environ[env]
#        int val
#        if attr in ('wait_command', 'history_limit', 'wait_slow_command',
#                    'num_close_matches'):
#            return int(val)
        if attr in ('debug'):
            return val.lower() == 'true'
        else:
            return val
    def _settings_from_env(self):
        """Loads settings from env."""
        return {attr: self._val_from_env(env, attr)
                for env, attr in const.ENV_TO_ATTR.items()
                if env in os.environ}

    def _settings_from_args(self, args):
        """Loads settings from args."""
        if not args:
            return {}

        from_args = {}
        if args.debug:
            from_args['debug'] = args.debug
        return from_args

settings = Settings(const.DEFAULT_SETTINGS)
