#!/bin/python3
import os
import subprocess
import shutil
import re
import filetype
import xdg
import time
from xdg.IconTheme import IconTheme
from xdg.DesktopEntry import DesktopEntry
from lmgbuild.lmgchecksum import LMGChecksum
#from lmgbuild.lmgutil import LMGUtil as utl
import lmgbuild.lmgutil as utl
import lmgbuild.logs as logs
from lmgbuild.lmgmeta import LMGMeta
from lmgbuild.lmgexceptions import *
LMG_BASE_DIR=["BaseInfo","Verify","Data","Data/AppData","Data/Source","Data/AppData/autostart"]
lmg_pkg_name=""
lmg_type=""
DEBUG=1


class LMGCreater:
    """lmg creater"""
    def __init__(self,lmg_type=None,lmg_output_dir=None,lmg_name=None,meta=None,lmg_base=None,lmg_release=None,lmg_workdir=None,lmg_base_dir=None,arch=None,deppath=None,msgque=None):
        self.createrversion = "1.0"
        self.creatername = 'LMGCreater'
        self.lmg_type = lmg_type
        self.lmg_output_dir = lmg_output_dir
        self.lmg_name = lmg_name
        self.lmg_base = lmg_base
        self.meta = meta
        self.lmg_release = lmg_release
        self.lmg_workdir = lmg_workdir
        self.lmg_base_dir=  lmg_base_dir
        self.arch = arch
        self.applayer = None
        self.deppath = deppath
        self.cleanimage =False
        self.cleanlayerimage = False
        self.cleanapplayer = False
        self.cleanbasedir = False
        self.cleandangling = False
        self.msgque = msgque
        self.lcffile = ["BaseInfo","Verify"]
        self.datafile = 'Data'
        self.containerfile = ""
    def set_pkgtype(self,pkgpath):
        self.msgque.put("set package type")
        kind = filetype.guess(pkgpath)
        if kind is None or kind.extension is None:
            self.pkgtype = ""
        else:
            self.pkgtype = kind.extension
    def set_lmg_output_dir(self,lmg_output_dir):
        self.lmg_output_dir=lmg_output_dir
    def set_lmg_base_dir(self,lmg_base_dir):
        self.lmg_base_dir=lmg_base_dir
    def set_lmg_meta(self,meta):
        self.meta = meta
    def set_lmg_base(self,lmg_base):
        self.lmg_base = lmg_base
    def set_lmg_type(self,lmgtype):
        """lmg type: BASRRUNTIME,COMPONENTS,APPLICATIONS"""
        self.lmg_type=lmgtype
    def set_lmg_release(self,lmg_release):
        self.lmg_release=lmg_release
    def get_lmg_type(self):
        return self.lmg_type
    def set_lmg_name_by_pkg(self,pkgpath):
        self.msgque.put("set lmg name")
        pkgname=os.path.basename(pkgpath)
        if self.lmg_type == "APPLICATIONS":
            self.set_lmg_name_by_deb(pkgname)
        elif self.lmg_type == "BASERUNTIME":
            self.set_lmg_name_by_tar(pkgname)
        elif self.lmg_type == "UPBASERUNTIME":
            self.set_lmg_name_by_upbase(pkgname)
    def set_lmg_name_by_deb(self,debversion):
        """
        Convert deb version to lmg version
        Argument: deb name
        """
        # check deb
        name,version,suffix = debversion.split("_")
        #    re_version = re_version.replace("deb10u","").replace("lnd.","")
#        if self.arch == "loong64":
#            suffix = suffix.replace("loongarch64","loong64").replace(".deb","")
#        else:
        suffix = suffix.replace(".deb","")
        #   for docker tag and repo
#        version = re.sub("[^A-Za-z0-9.]","-",version)
        #    lmg_version = up_version + "_" + LMG_VERSION
        #    lmg_pkg_name = name + "_" + lmg_version + "." + suffix
        self.appname = name
        self.appversion = version + "_" + self.lmg_release
        self.lmg_name = name + "_" + self.appversion + "." + suffix
        self.lmg_base_dir = self.lmg_workdir + "/" + self.lmg_name + ".app"
        self.apptag = self.appname + "_" + re.sub("[^A-Za-z0-9._-]","-",self.appversion) + ":" + self.lmg_base
        self.applayer = self.appname + "_" + re.sub("[^A-Za-z0-9._-]","-",self.appversion) + "." + self.lmg_base
        return self.lmg_name

    def set_lmg_name_by_tar(self,pkgname):
        "u20_1_1_brl.tar"
        self.appname,version,self.lmg_release,suffix = pkgname.split("_")
        #    re_version = re_version.replace("deb10u","").replace("lnd.","")
        suffix = suffix.replace(".tar","")
        #   for docker tag and repo
        self.appversion = version +"_" + self.lmg_release
        self.lmg_name = self.appname + "_" + self.appversion + "_" + suffix + "." + self.arch
        self.lmg_base_dir = self.lmg_workdir + "/" + self.lmg_name + ".app"
        self.apptag = self.appname + "_" + re.sub("[^A-Za-z0-9._-]","-",self.appversion) + ":" + "brl"
        self.applayer = self.appname +"_" + re.sub("[^A-Za-z0-9._-]","-",self.appversion) + "." + "brl"
        return self.lmg_name

    def set_lmg_name_by_upbase(self,pkgname):
        "uos20_1_1_up.1.tar"
        self.appname,version,self.lmg_release,suffix = pkgname.split("_")
        suffix = suffix.replace(".tar","")
        self.appversion = version +"_" + self.lmg_release + "_" + suffix
        self.lmg_name = self.appname + "_" + self.appversion + "." + self.arch
        self.lmg_base_dir = self.lmg_workdir + "/" + self.lmg_name + ".app"
        self.apptag = self.appname + "_" + re.sub("[^A-Za-z0-9._-]","-",self.appversion) + ":" + "brl"
        self.lmg_base = pkgname.split("_up")[0]
        self.applayer = self.appname +"_" + re.sub("[^A-Za-z0-9._-]","-",self.appversion) + "." + "brl"

    def create_lmg_basedir(self):
        """
        Create Lmg BaseDir
        Argument: lmg_app_dir
        """
        if not os.path.exists(self.lmg_base_dir):
            os.mkdir(self.lmg_base_dir)
            for path in LMG_BASE_DIR:
                path =  self.lmg_base_dir + "/" + path
                if not os.path.exists(path):
                    os.mkdir(path)
        self.cleanbasedir = True
        self.msgque.put("create lmgdir")
    def get_deb_baseinfo(self,debpath):
        control=utl.runshell(["dpkg-deb","-f",debpath]).stdout
        return control
    def get_baseinfo(self,pkgpath):
        kind = filetype.guess(pkgpath)
        if kind is None:
            return ""
        if kind.extension is "deb":
            return self.get_deb_baseinfo(pkgpath)
        elif kind.extension is "rpm":
            return "rpm"
        else:
            return ""
    def store_baseinfo(self,control):
        filename=self.lmg_base_dir + "/BaseInfo/control"
        with open(filename, "w") as f:
            f.write(control)
    def generate_baseinfo(self,pkgpath):
        self.msgque.put("generate baseinfo")
        control=self.get_baseinfo(pkgpath)
        self.store_baseinfo(control)
    def get_deb_normal_file_list(debname):
        #use ./export_normal.sh deb
        #get desktop:icon
        cmd = ["./export_source",debname]
        ret = runshell(cmd)
        if ret != 0 :
            raise ExportSourceError(cmd)
        filestr= ret.stdout
        filelist=[]
        for fpath in re.split(r"[:,\n]",filestr):
            if(fpath):
                filelist.append(fpath)
        return filelist
    def export_normal_files(filelist,destdir):
        """
        export desktop file,icon file to Data/AppData
        """
        for file in filelist:
            shutil.copy(file,destdir)
#    def get_deb_normal_file_list(debname):
#        cmd = ["dpkg","-L",debname]
#        ret = runshell(cmd)
#        if ret != 0 :
#            raise ExportSourceError(cmd)
#        filestr = ret.stdout
#        filelist = []
#        for f in filestr:
#            filelist.append(f)
#            entry = DesktopEntry(f)
#            iconname = entry.getIcon()
#            iconpath = IconTheme.getIconPath(iconpath)
#            filelist.append(iconpath)
#        return filelist
    def generate_meta(self,description=None):
        self.msgque.put("generate meta")
        self.create_meta(description)
        self.load_meta_cache()
        self.user_confirm_meta()
        self.store_meta()
    def create_meta(self,description):
        meta=LMGMeta(appname=self.appname,appversion=self.appversion,arch=self.arch,description=description,dependence=self.lmg_base,apptag=self.apptag,applayer=self.applayer,lmgtype=self.lmg_type)
        # load cache config
        #fix me
        self.meta=meta
    def load_meta_cache(self):
        lmghome=os.environ.get("LMGHOME")
        cachepath = lmghome + "/template/cache.yaml"
        stream=self.meta.get_stream_from_yaml(cachepath)
        self.meta.load_normal_info_from_stream(stream)

    def user_confirm_meta(self):
        #show metainfo
        #user change
        #self.meta.set_meta(key,value)
        pass
    def store_meta(self):
        metafile = self.lmg_base_dir + "/" + self.lmg_name + ".yaml"
        self.meta.save_meta(metafile)
        self.lcffile.append(os.path.basename(metafile))
    def generate_cmd_by_pkgtype(self,pkgpath):
        instpath = "/tmp/" + os.path.basename(pkgpath)
        if self.pkgtype is "deb":
            if self.lmg_base.startswith("uos"):
                cmd = "dpkg -i --force-all {} && rm -f {}".format(instpath,instpath)
            else:
                cmd = "apt install -y {} && rm -f {}".format(instpath,instpath)
            return cmd
        if self.pkgtype is "rpm":
            pass
        if self.pkgtype is "tar":
            pass
    def generate_dockerfile_precmd(self):
        depname = os.path.basename(self.deppath)
        preinstalldir = "/tmp/preinstall/" + os.path.basename(self.deppath)[0:-4]
        if self.deppath:
            cmd = "mkdir -p {}".format(preinstalldir) + " && " + \
                "cd /tmp" + " && " + \
                "tar xf {} -C {}".format(depname,preinstalldir) + " && " + \
                "cd {}".format(preinstalldir) + " && " + \
                "./preinstall.sh" + " && "
            return cmd
        else:
            return None
    def generate_dockerfile_by_template(self,dockerfile,base,cmd,pkg=None):
        """test"""
        template= "from " + base + "\n"
        if pkg:
            if not os.path.exists(os.path.basename(pkg)):
                shutil.copy(pkg,".")
            template = template + "COPY" + " " + os.path.basename(pkg) + " /tmp" + "\n"
        if self.deppath:
            if not os.path.exists(os.path.basename(self.deppath)):
                shutil.copy(self.deppath,".")
            template = template + "COPY" + " " + os.path.basename(self.deppath) + " /tmp" + "\n"
        template = template  + "RUN" +" " + cmd +";exit 0\n"
        with open(dockerfile,"w+") as f:
            f.writelines(template)

    def build_appimage_by_dockerfile(self,dockerfile,imagename,path="./"):
        logs.debug("Please wait for building image ...")
        logs.debug("check image ...")
        ret = utl.checkimage(imagename)
        if ret == 0:
            print(ret)
            logs.debug("Failed building image !!!")
            raise AppBuildError("Bug: imagename %s alraday exist,couldn't build" % imagename)
        os.chdir(os.path.dirname(dockerfile))
#        cmd = ["docker","build","-t",imagename,path,"--squash","--squashdelete","--no-cache"]
        cmd = ["loongker","build","-t",imagename,path,"--squash","--squashdelete","--no-cache"]
        ret = utl.runshell(cmd)
        if ret.returncode != 0:
            logs.debug("Failed building image !!!")
            raise AppBuildError(cmd)
        else:
            logs.debug("Complete building image !!!")
            self.cleanimage = True
    def export_appimage(self,imagename,path,baseimagename):
        logs.debug("Please wait for genning applayer ...")
#        cmd = ["docker","gen",imagename,self.applayer]
        cmd = ["loongker","gen",imagename,self.applayer]
        ret = utl.runshell(cmd)
        if ret.returncode != 0:
            raise AppExportError(cmd)
        else:
            logs.debug("Complete genning applayer !!!")
            self.cleanlayerimage = True
            self.cleanapplayer = True
#        cmd = ["docker","saveapp","-o",path,"-a",self.applayer]
        cmd = ["loongker","saveapp","-o",path,"-a",self.applayer]
        logs.debug("Please wait for saving image ...")
        ret = utl.runshell(cmd)
        if ret.returncode == 0:
            logs.debug("Complete Saving image !!!")
            self.containerfile=os.path.basename(path)
        else:
            raise AppExportError(cmd)
    def clean_container_by_imagename(self,imagename):
        logs.debug("clean container %s" % imagename.replace(':','-'))
#        cmd = ["docker","rm",imagename.replace(':','-')]
        cmd = ["loongker","rm",imagename.replace(':','-')]
        ret = utl.runshell(cmd)
        if ret.returncode != 0:
            raise CleanContainerError(cmd)
    def clean_image(self,apptag):
        logs.debug("clean image %s" % apptag)
        ret = 0
        if self.check_image(apptag) == 0:
#            cmd = ["docker","rmi",apptag]
            cmd = ["loongker","rmi",apptag]
            ret = utl.runshell(cmd)
            # to fix
        return ret
#        if ret.returncode != 0:
#            raise CleanImageError(cmd)
    def get_imageid_by_applayer(self,layername):
#        cmd = ["docker","images","-q",layername]
        cmd = ["loongker","images","-q",layername]
        ret = utl.runshell(cmd)
        imageid = ret.stdout
        if ret.returncode == 0:
            return imageid.strip()
        else:
            return 0
    def check_applayer(self,layername):
        pass

    def clean_applayer(self,layername):
        logs.debug("clean applayer %s" % layername)
#        cmd = ["docker","rma",layername]
        cmd = ["loongker","rma",layername]
        ret = utl.runshell(cmd)
#        if ret.returncode != 0:
#            raise CleanApplayerError(cmd)
    def run_container(self,imagename,appdir,command):
#        cmd = ["docker","run","-it","--rm","-v",appdir+":" +"/" + os.path.basename(appdir),imagename] + command
        cmd = ["loongker","run","-it","--rm","-v",appdir+":" +"/" + os.path.basename(appdir),imagename] + command
        ret = utl.runshell(cmd)
        if ret.returncode != 0:
            raise RunContainerError(cmd)
    def generate_checksum(self):
        self.msgque.put("generate_checksum")
        checkfile=self.lmg_base_dir + "/Verify/" + "checksum"
        checklmg=LMGChecksum()
        checksum=checklmg.GetFileContent(self.lmg_base_dir)
        checklmg.WriteToFile(checksum,checkfile)
    def signature_checksum(self,checkfile,signfile,sign):
        pass
#    def generate_lmg(self):
#        self.msgque.put("generate_lmg")
#        logs.debug("please wait for compressing...")
#        rootdir=os.getcwd()
#        tardir=os.path.dirname(self.lmg_base_dir)
#        os.chdir(tardir)
#        appdir=os.path.basename(self.lmg_base_dir)
#        ret=utl.runshell(["/usr/local/lmg/bin/tar","--zstd","-cf",self.lmg_output_dir+"/"+self.lmg_name + ".lmg",appdir],env={"path":"/usr/local/lmg/bin"}).returncode
#        if ret == 0:
#            logs.debug("complete building !!!")
#        else:
#            logs.warn("failed compressing !!!")
#        os.chdir(rootdir)
    def generate_lmg(self):
        self.msgque.put("generate_lmg")
        logs.debug("please wait for compressing...")
        rootdir=os.getcwd()
        tardir=os.path.dirname(self.lmg_base_dir)
        appdir=os.path.basename(self.lmg_base_dir)
        os.chdir(self.lmg_base_dir)
        #
        logs.debug(tardir)
        logs.debug(appdir)
        logs.debug("datafile")
        logs.debug(self.datafile)
        logs.debug("containerfile")
        logs.debug(self.containerfile)
        logs.debug("lcffile")
        logs.debug(self.lcffile)
        # meta file Verify baseinfo
        if os.path.exists('export_source'):
            os.remove('export_source')
#        utl.runshell(["/usr/local/lmg/bin/tar","--remove-files","-zcvf",self.lmg_output_dir+"/"+appdir+"/"+"data.tar.gz",self.datafile])
        utl.runshell(["tar","--remove-files","-zcvf","data.tar.gz",self.datafile])
#        utl.runshell(["/usr/local/lmg/bin/tar","--remove-files","-zcvf",self.lmg_output_dir+"/"+appdir+"/"+"lcf.tar.gz"] + self.lcffile)
        utl.runshell(["tar","--remove-files","-zcvf","lcf.tar.gz"] + self.lcffile)

        os.chdir(tardir)
#        ret=utl.runshell(["/usr/local/lmg/bin/tar","-cf",self.lmg_output_dir+"/"+self.lmg_name + ".lmg",appdir],env={"path":"/usr/local/lmg/bin"}).returncode
        ret=utl.runshell(["tar","-cf",self.lmg_output_dir+"/"+self.lmg_name + ".lmg",appdir]).returncode
        if ret == 0:
            logs.debug("complete building !!!")
        else:
            logs.warn("failed compressing !!!")
        os.chdir(rootdir)
    def check_image(self,apptag):
#        cmd = ["docker","inspect",apptag]
        cmd = ["loongker","inspect",apptag]
        ret = utl.runshell(cmd)
        return ret.returncode
    def checkdependence(self):
        "check dependence"
        self.msgque.put("checkdependence")
        if self.lmg_type != "APPLICATIONS" or self.lmg_base == None:
            return 0
        imagename = self.lmg_base + ":" +"brl"
#        cmd = ["docker","inspect",imagename]
        cmd = ["loongker","inspect",imagename]
        ret=utl.runshell(cmd)
        if ret.returncode:
            tarname=imagename.replace(":","_") + ".tar"
            if not os.path.exists(tarname):
                raise DependError("%s is not exists" % imagename)
            else:
                logs.warn("%s is not exists,auto install now" % imagename)
                self.installdependence(tarname)
        return 0
 #       p = subprocess.Popen(["docker","inspect",imagename],stderr=subprocess.PIPE,stdout=subprocess.PIPE)
 #       p.communicate()
 #       if p.returncode:
 #           print("%s is not exists install, this will autoinstall now" % imagename)
 #           self.lmg_installdependence(imagename)
 #       else:
 #           print("%s exists" %imagename)
 #           return 0
    def installdependence(self,tarname):
#        imagename.replace("\:","_") + ".loong64" + ".lmg"
#            cmd = ["docker","load","-i",tarname]
            cmd = ["loongker","load","-i",tarname]
            ret = utl.runshell(cmd)
            if ret.returncode:
                raise DependInstallError("%s is not exists" % tarname)
    def generate_data_app(self,pkgpath):
        if self.deppath:
            cmd = self.generate_dockerfile_precmd()
        else:
            cmd = ""
        cmd = cmd + self.generate_cmd_by_pkgtype(pkgpath)
        self.generate_dockerfile_by_template("./Dockerfile",self.lmg_base +":" +"brl",cmd,pkgpath)
        self.msgque.put("build image")
        self.build_appimage_by_dockerfile("./Dockerfile",self.meta.apptag)
#        shutil.copy("/usr/local/lmg/bin/export_normal.sh",self.lmg_base_dir)
        shutil.copy("/usr/local/lmg/bin/export_source",self.lmg_base_dir)
        self.msgque.put("run container")
        self.run_container(self.meta.apptag,self.lmg_base_dir, ["/" + os.path.basename(self.lmg_base_dir) + "/" + "export_source",self.meta.appname,os.path.basename(self.lmg_base_dir)])
        self.msgque.put("export image")
        self.export_appimage(self.meta.apptag,self.lmg_base_dir +"/" + self.lmg_name + ".tar",self.lmg_base +":" +"brl")
    def generate_data_base(self,pkgpath):
        self.msgque.put("save image")
        logs.debug("Please wait for saving ...")
        shutil.copy(pkgpath,self.lmg_base_dir)
        logs.debug("Complete saving !!!")
        self.msgque.put("tmp work")
        self.msgque.put("tmp work")
    def generate_data(self,pkgpath):
        if self.lmg_type == "APPLICATIONS":
            self.generate_data_app(pkgpath)
        elif self.lmg_type == "BASERUNTIME" or self.lmg_type == "UPBASERUNTIME":
            self.generate_data_base(pkgpath)
    def message_warnning():
        pass
    def message_warnning_gui():
        pass
    def clean_environ(self):
        logs.debug("Please wait for cleanning ...")
        self.msgque.put("clean environ")
        if self.cleanbasedir and not os.path.samefile(self.lmg_base_dir,"/"):
            logs.debug("clean build directory %s" % self.lmg_base_dir)
            cmd = ["rm","-rf",self.lmg_base_dir]
            ret = utl.runshell(cmd)
            if ret.returncode != 0:
                pass
        if self.lmg_type == "APPLICATIONS":
#            self.clean_container_by_imagename(self.meta.apptag)
            if self.cleanimage:
                self.clean_image(self.meta.apptag)
            if self.cleanlayerimage:
                imagenew = self.get_imageid_by_applayer(self.applayer)
                if imagenew:
                    self.clean_image(imagenew)
            if self.cleanapplayer:
                self.clean_applayer(self.applayer)
        # dangling image clean
        if self.cleandangling:
            logs.debug("clean dangling image")
#            cmd = ["docker","image","prune","-f"]
            cmd = ["loongker","image","prune","-f"]
            ret = utl.runshell(cmd)
            if ret.returncode != 0:
                pass
        self.msgque.put("end")
        logs.debug("Complete cleanning !!!")
#            raise CleanDanglingError(cmd)
        # lmg_base_dir clean
#                raise CleanDirError(cmd)
