#!/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function, unicode_literals
from pprint import pprint
from PyInquirer import prompt, Separator
from prompt_toolkit.validation import Validator, ValidationError
from examples import custom_style_2
import regex
import os
import filetype
import yaml
import platform
import sys
from collections import namedtuple
import re
#from lmgbuild.lmgutil import LMGUtil as utl
import lmgbuild.lmgutil as utl
from lmgbuild.lmgexceptions import *
DEBUG=0




Preset = namedtuple('Preset','lmg_type lmg_output_dir lmg_workdir arch lmg_release lmg_base')
class LmgRelaseValidator(Validator):
    def validate(self, document):
        ok = regex.match("^[1-9]\d*$",document.text)
        if not ok:
            raise ValidationError(
                message='Please enter a valid lmg_release number',
                cursor_position=len(document.text))  # Move cursor to end

class LmgPathValidator(Validator):
    def validate(self, document):
        pathdir = utl.formatpath(document.text)
        ok = os.path.isdir(pathdir)
        if ok:
            ok = os.access(pathdir, os.X_OK | os.W_OK)
            if not ok:
                raise ValidationError(
                    message='Permission denied for directory',
                    cursor_position=len(document.text))  # Move cursor to end
        else:
            raise ValidationError(
                message='Please enter a valid directory',
                cursor_position=len(document.text))  # Move cursor to end

class LmgFileValidator(Validator):
    def validate(self,document):
        ok = os.path.isfile(document.text)
        if not ok:
            raise ValidationError(
                message='Please enter a valid filepath',
                cursor_position=len(document.text))  # Move cursor to end
class LmgDebValidator(Validator):
    def validate(self,document):
        debpath = utl.formatpath(document.text)
        ok = os.path.isfile(debpath)
        if not ok:
            raise ValidationError(
                message='Filepath is not a file ,Please enter a valid filepath',
                cursor_position=len(document.text))  # Move cursor to end
        kind = filetype.guess(debpath)
        if kind is None or kind.extension is not "deb":
            raise ValidationError(
                message='The file is not deb file,Please input a deb filepath . Rpm/tar is not support now',
                cursor_position=len(document.text))  # Move cursor to end
class LmgPkgValidator(Validator):
    def validate(self,document):
        pkgpath = utl.formatpath(document.text)
        ok = os.path.isfile(pkgpath)
        if not ok:
            raise ValidationError(
                message='Filepath is not a file ,Please enter a valid filepath',
                cursor_position=len(document.text))  # Move cursor to end
        kind = filetype.guess(pkgpath)
        if kind is None or (kind.extension is not "deb" and kind.extension is not "rpm" and kind.extension is not "tar"):
            raise ValidationError(
                message='The file is not package file,Please input a deb/rpm/tar filepath',
                cursor_position=len(document.text))  # Move cursor to end
class LmgTarValidator(Validator):
    def validate(self,document):
        tarpath = utl.formatpath(document.text)
        ok = os.path.isfile(tarpath)
        if not ok:
            raise ValidationError(
                message='Filepath is not a file ,Please enter a valid filepath',
                cursor_position=len(document.text))  # Move cursor to end
        kind = filetype.guess(tarpath)
        if kind is None or kind.extension is not "tar":
            raise ValidationError(
                message='The file is {} file,Please input a tar filepath'.format(kind.extension),
                cursor_position=len(document.text))  # Move cursor to end
class LmgUpTarValidator(Validator):
    def validate(self,document):
        tarpath = utl.formatpath(document.text)
        ok = os.path.isfile(tarpath)
        if not ok:
            raise ValidationError(
                message='Filepath is not a file ,Please enter a valid filepath',
                cursor_position=len(document.text))  # Move cursor to end
        kind = filetype.guess(tarpath)
        if kind is None or kind.extension is not "tar":
            raise ValidationError(
                message='The file is {} file,Please input a tar filepath'.format(kind.extension),
                cursor_position=len(document.text))  # Move cursor to end
        uptarname = os.path.basename(tarpath)
        ret = re.match(r"\w+_\d+_\d+_up.\d+.tar",uptarname)
        if not ret:
            raise ValidationError(
                message='The file not a upbaseruntime package',
                cursor_position=len(document.text))
class LmgBaseTarValidator(Validator):
    def validate(self,document):
        tarpath = utl.formatpath(document.text)
        ok = os.path.isfile(tarpath)
        if not ok:
            raise ValidationError(
                message='Filepath is not a file ,Please enter a valid filepath',
                cursor_position=len(document.text))  # Move cursor to end
        kind = filetype.guess(tarpath)
        if kind is None or kind.extension is not "tar":
            raise ValidationError(
                message='The file is {} file,Please input a tar filepath'.format(kind.extension),
                cursor_position=len(document.text))  # Move cursor to end
        basetarname = os.path.basename(tarpath)
        ret = re.match(r"\w+_\d+_\d+_brl.tar",basetarname)
        if not ret:
            raise ValidationError(
                message='The file not a baseruntime package',
                cursor_position=len(document.text))
class LmgDepTarValidator(Validator):
    def validate(self,document):
        tarpath = utl.formatpath(document.text)
        if tarpath:
            ok = os.path.isfile(tarpath)
            if not ok:
                raise ValidationError(
                    message='Filepath is not a file ,Please enter a valid filepath',
                    cursor_position=len(document.text))  # Move cursor to end
            kind = filetype.guess(tarpath)
            if kind is None or (kind.extension is not "tar" and kind.extension is not "gz"):
                raise ValidationError(
                    message='The file is {} file,Please input a tar filepath'.format(kind.extension),
                    cursor_position=len(document.text))  # Move cursor to end
class FactoryPreseter:
    def __init__(self):
        self.init_env()
    def product_preseter(self):
        self.get_runtime_conf()
        lmgtype,lmg_output_dir,lmg_work_dir =self.init_preset()
        if DEBUG:
            print("-----------start init_preset------------------")
            print(lmgtype)
            print(lmg_output_dir)
            print(lmg_work_dir)
            print("-----------end init_preset------------------")
        if lmgtype == "BASERUNTIME":
            return BASEPRESET(lmgtype,lmg_output_dir,lmg_work_dir)
        elif lmgtype == "UPBASERUNTIME":
            return UPBASEPRESET(lmgtype,lmg_output_dir,lmg_work_dir)
        elif lmgtype == "APPLICATIONS":
            return APPPRESET(self.runtime_dic,lmgtype,lmg_output_dir,lmg_work_dir)
    def init_env(self):
        runtimeconfs = ["conf/default-base.yaml","conf/local-base.yaml","conf/repository-base.yaml"]
        self.lmghome = os.environ.get("LMGHOME")
        if self.lmghome is None:
            if os.path.exists(".env.yaml"):
                with open(".env.yaml",'r') as f:
                    stream = yaml.safe_load(f)
                    if stream is not None:
                        self.lmghome = stream['LMGHOME']
                        os.environ['LMGHOME']=self.lmghome
        if self.lmghome is None:
            self.lmghome = os.environ.get("HOME") + "/.config/lmgbuild"
            os.environ['LMGHOME']=self.lmghome
            #print(os.environ.get("LMGHOME"))
        self.runtimeconfs = list(map(lambda x: self.lmghome + "/" + x,runtimeconfs))
        self.generate_cache(self.lmghome)
    def generate_cache(self,workdir):
        #template/cache.yaml
        cachedir = workdir + "/template"
        cachefile = cachedir + "/cache.yaml"
        if not os.path.exists(cachedir):
            os.makedirs(cachedir)
            if not os.path.exists(cachefile):
                cache=dict()
                cache['contact'] = {"bugreport": "www.lmg-bug.com", "email": "lmg@loongson.cn"}
                cache['generate'] = {"genfrom":"manual","tool":{"name":"lmgbuild","version":'1.0'}}
                cache['parallel'] = False
                cache['permission'] = "X11 Sound IM-input Dbus Camera Cups Xdg"
                cache['publisher'] = {"email":"lmg@loongson.cn","name":"lmg"}
                with open(cachefile,"w") as f:
                    yaml.dump(cache,f,allow_unicode=True)
    def get_runtime_conf(self):
        self.supportarch = []
        self.runtime_dic = {}
#        utl.sync_repository(tryexec=True)
        for path in self.runtimeconfs:
            if not os.path.exists(path):
                if not os.path.exists(os.path.dirname(path)):
                    os.makedirs(os.path.dirname(path))
                os.mknod(path)
                continue
            with open(path,'r') as f:
                stream = yaml.safe_load(f)
                if stream is None:
                    continue
                self.supportarch += list(stream.keys())
                self.supportarch = list(dict.fromkeys(self.supportarch))
                for arch in self.supportarch:
                    for item in stream.setdefault(arch,[]):
                        if item not in self.runtime_dic.setdefault(arch,[]):
                           self.runtime_dic.setdefault(arch,[]).append(item)
    def init_preset(self):
        init_questions = [
            {
                'type': 'list',
                'name': 'lmg_type',
                'message': 'Which type lmg do you want to make?',
                'choices': [
                    'BASERUNTIME',
                    'UPBASERUNTIME',
                    'APPLICATIONS',
                    {
                        'name':'COMPONENTS',
                        'disabled':'not supopport at now'
                    }
                ]
            },
            {
                'type': 'input',
                'name': 'lmg_output_dir',
                'message': 'Which directory do you want to save the lmg package?',
                'default': '/tmp',
                'validate': LmgPathValidator
            },
            {
                'type': 'input',
                'name': 'lmg_work_dir',
                'message': 'Which directory do you want to build the lmg package?',
                'default': '/tmp',
                'validate': LmgPathValidator
            },
        ]
        preenv = prompt(init_questions,style=custom_style_2)
        return utl.formatpath(preenv['lmg_type']),utl.formatpath(preenv['lmg_output_dir']),utl.formatpath(preenv['lmg_work_dir'])

class BASEPRESET():
    def __init__(self,lmg_type,lmg_output_dir,lmg_work_dir):
        self.lmg_type = lmg_type
        self.lmg_output_dir = lmg_output_dir
        self.lmg_work_dir = lmg_work_dir
    def preset(self):
        base_questions = [
            {
                'type': 'list',
                'name': 'arch',
                'message': 'Please check your arch ?',
                'choices': ['loongarch64','x86','mips64el','arch64'],
                'default': 'loongarch64'
            },
            {
                'type': 'input',
                'name': 'tarpath',
                'message': 'Which tar package do you want to convert?',
                'validate': LmgBaseTarValidator
            },
            {
                'type': 'input',
                'name': 'description',
                'message': 'Please input the description for this BASERUNTIME lmg package'
            }
        ]
        baseenv = prompt(base_questions,style=custom_style_2)
        self.arch = utl.formatpath(baseenv['arch'])
        self.lmg_release = ""
        self.lmg_base = ""
        preset=Preset(self.lmg_type,self.lmg_output_dir,self.lmg_work_dir,self.arch,self.lmg_release,self.lmg_base)
        return preset,utl.formatpath(baseenv['tarpath']),utl.formatpath(baseenv['description']),""

class UPBASEPRESET():
    def __init__(self,lmg_type,lmg_output_dir,lmg_work_dir):
        self.lmg_type = lmg_type
        self.lmg_output_dir = lmg_output_dir
        self.lmg_work_dir = lmg_work_dir
    def preset(self):
        base_questions = [
            {
                'type': 'list',
                'name': 'arch',
                'message': 'Please check your arch ?',
                'choices': ['loongarch64','x86','mips64el','arch64'],
                'default': 'loongarch64'
            },
            {
                'type': 'input',
                'name': 'tarpath',
                'message': 'Which tar package do you want to convert?',
                'validate': LmgUpTarValidator
            },
            {
                'type': 'input',
                'name': 'description',
                'message': 'Please input the description for this BASERUNTIME lmg package'
            }
        ]
        baseenv = prompt(base_questions,style=custom_style_2)
        self.arch = utl.formatpath(baseenv['arch'])
        self.lmg_release = ""
        self.lmg_base = ""
        preset=Preset(self.lmg_type,self.lmg_output_dir,self.lmg_work_dir,self.arch,self.lmg_release,self.lmg_base)
        return preset,utl.formatpath(baseenv['tarpath']),utl.formatpath(baseenv['description']),""
class APPPRESET():
    def __init__(self,runtime_dic,lmg_type,lmg_output_dir,lmg_work_dir):
        self.runtime_dic = runtime_dic
        self.lmg_type = lmg_type
        self.lmg_output_dir = lmg_output_dir
        self.lmg_work_dir = lmg_work_dir
    def preset(self):
#        utl.sync_repository()
        arch = platform.machine()
#        if arch == "loongarch64":
#            arch = "loong64"
        if self.runtime_dic.setdefault(arch,"") == "" :
            #print(self.runtime_dic)
            raise NoRuntimeError("please import baseruntime for %s or run 'lmgbuild -s/--sync' to sync repository config" % arch)
        app_questions1 = [
            {
                'name': 'lmg_base',
                'type': 'list',
                'message': 'Which BASRRUNTIME you dependent?',
                'choices': self.runtime_dic[arch]
            },
        ]

        app_questions2 = [
            {
                'type': 'confirm',
                'name': 'download',
                'message': 'Do you want to install baseruntime?',
                'default': True
            },
        ]
        appenv1 = prompt(app_questions1, style=custom_style_2)
        #check
        self.lmg_base = utl.formatpath(appenv1['lmg_base'])

        app_questions3 = [
            {
                'type': 'input',
                'name': 'lmg_release',
                'message': 'Which release do you want?',
                'default': '1',
                'validate': LmgRelaseValidator
            },
            {
                'type': 'input',
                'name': 'pkgpath',
# not support rpm/tar
#                'message': 'Which  package do you want to convert?',
                'message': 'Which deb package do you want to convert?',
                'validate': LmgDebValidator
#                'validate': LmgPkgValidator
            },
            {
                'type': 'input',
                'name': 'deppath',
                'message': 'Input your depend tar package or input Enter?',
                'validate': LmgDepTarValidator
            },
        ]
        appenv3 = prompt(app_questions3, style=custom_style_2)
        self.lmg_release = utl.formatpath(appenv3['lmg_release'])

        ret = utl.checkdependence(appenv1['lmg_base'])
        if ret:
            appenv2 = prompt(app_questions2, style=custom_style_2)
            if appenv2['download']:
                utl.auto_installbase(appenv1['lmg_base'],arch)
#                ret3=utl.getdependence(appenv1['lmg_base'],arch)
#                if ret3 == 0:
#                    ret4=utl.import_baseruntime(appenv1['lmg_base'] +"_brl.tar")
#                    if ret4 != 0:
#                        raise DependInstallError("install baseruntime error")
#                elif ret3 == -1:
#                    raise DownloadBaseError("no baseruntime %s in repository,please use \"lmgbuild -i/--import [baseruntime]\" import the baseruntime" % appenv1['lmg_base'])
#                else:
#                    raise DownloadBaseError("Download baseruntime error")
#            else:
#                print("no baseruntime,please install ....")
#                raise DependError("%s is not exists" % appenv1['lmg_base'])

        preset=Preset(self.lmg_type,self.lmg_output_dir,self.lmg_work_dir,arch,self.lmg_release,self.lmg_base)

#        description = yaml.safe_load(utl.runshell(["dpkg-deb","-f",appenv3['pkgpath']]).stdout)['Description']
        description = "lmg app"
        return preset,utl.formatpath(appenv3['pkgpath']),utl.formatpath(description),utl.formatpath(appenv3['deppath'])
#pprint(lmgset)
#apppreset=prompt(applications_preset,style=custom_style_2)
#pprint(appset)
