#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import attr
import logging

from lmgrun.lmgyaml import LMGYaml

CONFIG_HOME_DIR = '/var/lib/lpkg/'
log = logging.getLogger(__name__)

class BaseInfo(object):
    '''
    '''
    cmd            = []
    app_name       = '<none>'
    parallel       = '<none>'
    cname          = '<none>'
    permission     = '<none>'
    img_name       = '<none>'
    cntr_name      = '<none>'

    def __init__(self, cmdline):
        BaseInfo.cmd = cmdline.args.cmd
        self.pkg_name = cmdline.args.name
        self.get_img_name()
        self.get_cntr_name()
        self.get_app_info()
        log.debug("--------------------------------------------")

    def get_img_name(self):
        config_file = os.path.join(CONFIG_HOME_DIR, 'appindex.cfg')
        ly = LMGYaml(config_file)
        BaseInfo.img_name, self.app_cfg_file = ly.find_file_location(self.pkg_name)
        log.debug("pkg_ame:%s img_name:%s yamlfile:%s", self.pkg_name, BaseInfo.img_name, self.app_cfg_file)
        if self.app_cfg_file is None and BaseInfo.img_name is None:
            log.error("The yaml file for the %s package does not exist", self.pkg_name)
            raise Exception("Not Found yaml file!")

    def get_cntr_name(self):
        """
        Get the name for the app's container.
        """
        realbin=os.path.split(self.cmd[0])[1]
        BaseInfo.cntr_name = "{}_{}".format(BaseInfo.img_name.replace(":", "_"), realbin)

    def get_app_info(self):
        ly = LMGYaml(self.app_cfg_file, False)
        BaseInfo.app_name  = ly.get_value('appname')
        BaseInfo.parallel = ly.get_value('parallel')
        BaseInfo.cemail = ly.get_value('contact')['email']
        permission = ly.get_value('permission')
        BaseInfo.permission = list(filter(None, permission.split(" ")))

class AppRunInfo(BaseInfo):
    '''
    '''
    def __init__(self):
        self.volumes         = []
        self.env             = []
        self.devices         = []
        self.uid             = str(os.getuid())
        self.xdg_runtime_dir = str(os.getenv('XDG_RUNTIME_DIR'))
        self.qim             = str(os.getenv('QT_IM_MODULE'))
        self.cim             = str(os.getenv('CLUTTER_IM_MODULE'))
        self.gim             = str(os.getenv('GTK_IM_MODULE'))
        self.q4im            = str(os.getenv('QT4_IM_MODULE'))
        self.xmod            = str(os.getenv('XMODIFIERS'))
        self.xdd             = os.getenv('XDG_DATA_DIRS')
        self.xdh             = os.getenv('XDH_DATA_HOME')
        self.prepare_volumes()
        self.prepare_environment()
        self.prepare_devices()
    #挂载设备
    def switch_to_case_device(self, argument: str):
        switcher = {
            "SOUND" : ['-v', '/dev/snd:/dev/snd', '-v', '/dev/dri:/dev/dri', '--device', '/dev/snd:/dev/snd'],
            "CAMERA": ['-v', '/dev/video0:/dev/video0', '-v', '/dev/shm:/dev/shm'],
        }
        return switcher.get(argument, "nothing")

    def prepare_devices(self):
        for it in BaseInfo.permission:
            arg = self.switch_to_case_device(it.upper())
            if arg == 'nothing' :
                log.debug("no permission: %s in device directory", it)  # Prints String are equal
            else :
                self.devices += arg
    #环境变量信息
    def switch_to_case_env(self, argument: str):
        #xdd = os.getenv('XDG_DATA_DIRS')
        #xdh = os.getenv('XDH_DATA_HOME')
        xdg = []
        if self.xdh != None :
            xdg.append('-e')
            xdg.append('XDG_DATA_HOME=' + self.xdh)

        if self.xdd != None :
            xdg.append('-e')
            xdg.append('XDG_DATA_DIRS=' + self.xdd)

        switcher = {
            "SOUND"    : ['-e', 'PULSE_SERVER=unix:'+self.xdg_runtime_dir+'/pulse/native'],
            "X11"      : ['-e', 'DISPLAY='+os.getenv('DISPLAY'), '-e', 'XDG_RUNTIME_DIR='+self.xdg_runtime_dir],
            "IM-INPUT" : ['-e', 'QT_IM_MODULE=' + self.qim, '-e', 'CLUTTER_IM_MODULE=' + self.cim, '-e', 'GTK_IM_MODULE=' + self.gim, '-e', 'QT4_IM_MODULE=' + self.q4im, '-e', 'XMODIFIERS=' + self.xmod],
            "DBUS"     : ['-e', 'DBUS_SESSION_BUS_ADDRESS='+os.environ['DBUS_SESSION_BUS_ADDRESS']],
            "XDG"      : xdg,
            #"XDG"      : ['-e', 'XDG_CURRENT_DESKTOP='+os.getenv('XDG_CURRENT_DESKTOP'),
            # '-e', 'XDG_DATA_DIRS='+os.getenv('XDG_DATA_DIRS'), '-e',
            # 'XDG_GREETER_DATA_DIR='+os.getenv('XDG_GREETER_DATA_DIR'), '-e',
            # 'XDG_SEAT='+os.getenv('XDG_SEAT'),  '-e', 'XDG_SEAT_PATH='+os.getenv('XDG_SEAT_PATH'),
            # '-e', 'XDG_SESSION_CLASS='+os.getenv('XDG_SESSION_CLASS'),
            # '-e', 'XDG_SESSION_DESKTOP='+os.getenv('XDG_SESSION_DESKTOP'),
            # '-e', 'XDG_SESSION_ID='+os.getenv('XDG_SESSION_ID'),
            # '-e', 'XDG_SESSION_PATH='+os.getenv('XDG_SESSION_PATH'),
            # '-e', 'XDG_SESSION_TYPE='+os.getenv('XDG_SESSION_TYPE'),
            # '-e', 'XDG_VTNR='+os.getenv('XDG_VTNR')],
        }
        #log.debug("=======================%s" % switcher)

        return switcher.get(argument, "nothing")

    def prepare_environment(self):
        """
        Prepare environment variables for a app container.
        """
        for it in BaseInfo.permission:
            arg = self.switch_to_case_env(it.upper())
            if arg == 'nothing' :
                log.debug("no permission:%s in env directory", it)  # Prints String are equal
            else:
                self.env += arg
    #挂载总线，X11，cpu
    def switch_to_case_volume(self, argument: str):
        switcher = {
            "X11"   : ['-v', '/tmp/.X11-unix:/tmp/.X11-unix:ro', '-v', self.xdg_runtime_dir+':/run/user/'+self.uid],
            "CUPS"  : ['-v', '/var/run/cups:/var/run/cups:ro'],
            "DBUS"  : ['-v', '/var/run/dbus/system_bus_socket:/var/run/dbus/system_bus_socket', '-v', self.xdg_runtime_dir+'/bus:/run/user/'+self.uid+'/bus']
        }
        return switcher.get(argument, "nothing")

    def prepare_volumes(self):
        for it in BaseInfo.permission:
            arg = self.switch_to_case_volume(it.upper())
            if arg == 'nothing' :
                log.debug("no permission:%s in volume directory", it)  # Prints String are equal
            else :
                self.volumes += arg
