#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging
import subprocess
from time import sleep

from lmgrun.execute import execute
from lmgrun.baseinfo import BaseInfo
from lmgrun.exceptions import HotStartError, ColdStartError, ContainerRmError, ParallelStartError

log = logging.getLogger(__name__)

class CntrRun(BaseInfo):
    """
    Class for accessing information about a app's container.
    """
    def __init__(self):
        self.img_name  = BaseInfo.img_name
        self.cntr_name = BaseInfo.cntr_name
        self.cmd       = BaseInfo.cmd

    def app_cold_start(self, run_args: list):
        try:
            res = self.run(run_args)
            log.debug("Successfully cold started container: %s, res: %s",  self.cntr_name, res)
        except ColdStartError:
            log.error("Failed to hot start container:%s", self.cntr_name)

    def app_hot_start(self):
        try:
            res = self.start()
            log.debug("Successfully hot started container: %s, res:%s", self.cntr_name, res)
        except HotStartError:
            log.error("Failed to hot start container:%s", self.cntr_name)

    def app_parallel_start(self):
        try:
            res = self.exec(self.cmd)
            log.debug("Successfully exec started container: %s, res:%s",  self.cntr_name, res)
        except ParallelStartError:
            log.error("Failed to parallel start container :%s", self.cntr_name)

    def app_uninstall_container(self):
        try:
            res = self.stop()
            log.debug("Successfully rm container: %s, res: %s", self.cntr_name, res)
        except ContainerRmError:
            log.error("Failed to rm container:%s", self,cntr_name)

    def run(self, run_args: list):
        """
        run container
        """
        runcmd = ['loongker', 'run'] + run_args + ['--name', self.cntr_name, self.img_name] + self.cmd
        result = execute(runcmd)
        if not result.succeeded:
            #raise ColdStartError(result.command)
            raise ColdStartError(result.__dict__)
        return result.out.strip()

    def start(self):
        """
        start container
        """
        runcmd=['loongker', 'start']
        runcmd.append(self.cntr_name)
        result = execute(runcmd)
        if not result.succeeded:
            #raise HotStartError(result.command)
            raise HotStartError(result.__dict__)
        return result.out.strip()

    def stop(self):
        """
        kill container
        rm container
        """
        """loongker kill <container name>"""
        runcmd=['loongker', 'kill']
        runcmd.append(self.cntr_name)
        execute(runcmd)
        """loongker rm <container name>"""
        runcmd[1] = 'rm'
        result = execute(runcmd)
        if not result.succeeded:
            raise ContainerRmError(result.command)
        return result.out.strip()

    def exec(self, command: list):
        """
        exec container
        """
        runcmd=['loongker', 'exec',  '-d']
        runcmd.append(self.cntr_name)
        runcmd +=command
        result = execute(runcmd)
        if not result.succeeded:
            raise ParallelStartError(result.command)
        return result.out.strip()
