#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging

from lmgrun import lmgwin
from lmgrun.feedback import feedback
from lmgrun.lmgenum import StatusCodeEnum

log = logging.getLogger(__name__)

class appException(Exception):
    def __init__(self, message, data):
        log_msg = '[{}] {}'.format(data['code'], message)
        prompt_msg = 'Error code:{} {}' .format(data['code'], data['errmsg'])
        log.error(log_msg)
        lmgwin.lmg_prompt('error', prompt_msg)
       # feedback()

class ContainerInfoError(appException):
    def __init__(self, command):
        data = {
            'code': StatusCodeEnum.APP_CONTAINER_INFO_ERR.code,
            'errmsg': StatusCodeEnum.APP_CONTAINER_INFO_ERR.errmsg
        }
        message = "inspect error: ({})".format(command)
        super(ContainerInfoError, self).__init__(message, data)

class HotStartError(appException):
    def __init__(self, reason):
        data = {
            'code': StatusCodeEnum.APP_HOT_START_ERR.code,
            'errmsg': StatusCodeEnum.APP_HOT_START_ERR.errmsg
        }
        message = "hot start error: ({})".format(command)
        super(HotStartError, self).__init__(message, data)

class ColdStartError(appException):
    def __init__(self, command):
        data = {
            'code': StatusCodeEnum.APP_COLD_START_ERR.code,
            'errmsg': StatusCodeEnum.APP_COLD_START_ERR.errmsg
        }
        message = "cold start error:({})".format(command)
        super(ColdStartError, self).__init__(message, data)

class ParallelStartError(appException):
    def __init__(self, command):
        data = {
            'code': StatusCodeEnum.APP_PARALLEL_START_ERR.code,
            'errmsg': StatusCodeEnum.APP_PARALLEL_START_ERR.errmsg
        }
        message = "parallel start error:({})".format(command)
        super(ParallelStartError, self).__init__(message, data)

class ContainerRmError(appException):
    def __init__(self, command):
        data = {
            'code': StatusCodeEnum.CONTAINER_RM_ERR.code,
            'errmsg': StatusCodeEnum.CONTAINER_RM_ERR.errmsg
        }
        message = "rm container error:({})".format( command)
        super(ContainerRmError, self).__init__(message, data)

class LoongkerImageNotFound(appException):
    def __init__(self, image_name):
        data = {
            'code': StatusCodeEnum.IMAGE_ERR.code,
            'errmsg': StatusCodeEnum.IMAGE_ERR.errmsg
        }
        message = "loongker image:({}) don't exists".format(image_name)
        super(LoongkerImageNotFound, self).__init__(message, data)
