#! /usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
import time
import psutil
import logging
import platform

from lmgrun import lmgwin
from lmgrun.lmglog import LMGLog
from lmgrun.baseinfo import BaseInfo
from lmgrun.mail import sendMail

log = logging.getLogger(__name__)
megabyte = float(1024 * 1024)

class Feedback(object):
    def __init__(self):
        self.context = '<none>'
        self.get_file()

    def get_file(self):
        self.log_path = os.environ['HOME'] + '/.lmg/lmgrun/'
        log_name = time.strftime("%Y-%m-%d", time.localtime()) + '_' + ''.join(BaseInfo.cntr_name ) + '.log'
        self.log_file_path = os.path.join(self.log_path, log_name)

    def write_context_to_file(self):
        self.system_details()
        self.ram_details()
        self.write_to_file()

    # Defining system_details function.
    def system_details(self):
        os_type = platform.system()
        self.context += 'OS Type: ' + platform.system() + "\n"
        self.context += 'OS Release version: ' + platform.release() + "\n"
        self.context += 'Processor Name: ' + platform.processor() + "\n"
        self.context += 'Machine Name on Network is: ' + platform.node() + "\n"
        sself.context += 'Machine Type is: ' + platform.Machine() + "\n"
        self.context += 'System platform type is: ' + sys.platform

    def ram_details(self):
        # Assign variable to store total memory value.
        mem = psutil.virtual_memory()
        mem_total = float(mem.total/megabyte)

        # Assign variable with the value of currently available memory.
        mem_free = float(mem.free/megabyte)

        # Assign variable with the value of currently used memory.
        mem_used = float(mem.used/megabyte)
        self.context += 'Total memory is: ' + str(round(float(mem_total),2)) + ' MBs' + '\n'
        self.context += 'Current available memory is: ' + str(round(float(mem_free),2)) + ' MBs' + '\n'
        self.context += 'Current used memory is: ' + str(round(float(mem_used),2)) + ' MBs' + '\n'
        self.context += 'Percentage of RAM being utilized currently: ' + str(round(float(mem.percent),2)) + '%'

    def write_to_file(self):
        self.fb_file_path = '/tmp/feedback_' + BaseInfo.cntr_name
        with open(self.fb_file_path, 'w') as fw: # 打开文件
            fw.writelines([self.context,  '\n'])

    def file_remove(self):
        os.remove(self.fb_file_path)

def feedback():

    confirm = lmgwin.lmg_hint('yesno', '程序运行出现问题，是否需要上报?(yes/no)')

    if confirm == False:
        sys.exit(0)

    files_list = []
    fb = Feedback()

    if os.path.exists(fb.log_file_path):
        sz = os.path.getsize(fb.log_file_path)
        if not sz:
            log.debug("log file is empty")
        files_list.append(fb.log_file_path)
    else:
        lmgwin.lmg_prompt('info', '无法获得日志信息，请先运行应用')
        log.debug("files_list is empty, should run app first")
        sys.exit(0)

    auth =  False
    join = lmgwin.lmg_hint('yesno', '是否授权lmgrun程序获取系统信息?(yes/no)')
    if join == True:
        fb.write_context_to_file()
        files_list.append(fb.fb_file_path)
        auth = True
    elif join == False:
        auth = False

    sendMail(files_list)

    if auth == True:
        fb.file_remove()

    sys.exit(0)
