#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import logging

from lmgrun.execute import execute
from lmgrun.baseinfo import BaseInfo
from lmgrun.exceptions import ContainerInfoError

log = logging.getLogger(__name__)

class Inspector(BaseInfo):
    def __init__(self):
        self.img_name = BaseInfo.img_name
        self.cntr_name = BaseInfo.cntr_name

    def inspect(self, option1, option2):
        """
        Return the full container status from Loongker.
        """
        runcmd=['loongker', 'inspect', '--format', '{{.'+option1+'.'+option2+'}}']
        runcmd.append(self.cntr_name)
        result = execute(runcmd)
        if not result.succeeded:
            raise ContainerInfoError(result.command)
        return result.out.strip()

    def get_status(self):
        """
        Return the status of the container (running, exited, paused).
        Returns "missing" if the app does not exist.
        """
        try:
            info = self.inspect('State', 'Status')
            return info
        except ContainerInfoError:
            log.error("get %s status error.", self.cntr_name)

    def get_exited_code(self):
        """
        Return the exited code of the container (running, exited, paused).
        Returns "missing" if the app does not exist.
        """
        try:
            info = self.inspect('State', 'ExitCode')
            return info
        except ContainerInfoError:
            log.error("get %s exited code error.", self.cntr_name)

    def get_command(self):
        """
        Return the cmd of the container (running, exited, paused).
        Returns "missing" if the app does not exist.
        """
        try:
            info = self.inspect('Config', 'Cmd')
            return info
        except ContainerInfoError:
            log.error("get %s command error.", self.cntr_name)
