#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Desc: { 项目日志类模块 }

import os
import sys
import time
import logging

from lmgrun.baseinfo import BaseInfo

#CRITICAL = 50 #FATAL = CRITICAL # 紧急
#ERROR = 40	# 错误
#WARNING = 30 #WARN = WARNING 	# 警告
#INFO = 20	# 消息
#DEBUG = 10  # 调试
#NOTSET = 0  # 不设置
class LMGLog(BaseInfo):

    level_relations = {
        'debug':logging.DEBUG,
        'info':logging.INFO,
        'warning':logging.WARNING,
        'error':logging.ERROR,
        'crit':logging.CRITICAL
    }
    def __init__(self, level = "info", use_console = True):
        """
        level： 设置日志的打印级别，默认为DEBUG
        use_console： 是否在控制台打印，默认为True
        """
        self._format = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
        self.get_level(level)
        self.get_file()

        self.logger = logging.getLogger()
        self.logger.setLevel(logging.DEBUG)
        self.set_formatter_file()

        if use_console:
            self.set_formatter_stream()

    def addHandler(self, func):
        self.logger.addHandler(func)

    def get_level(self, level):
        self._level = self.level_relations.get(level.lower())

    def get_file(self):
        self.log_path = os.environ['HOME'] + '/.lmg/lmgrun/'
        log_name = time.strftime("%Y-%m-%d", time.localtime()) + '_' + ''.join(BaseInfo.img_name.replace(":", "_") ) + '.log'
        self.log_file_path = os.path.join(self.log_path, log_name)

    def set_formatter_file(self):
        if not os.path.exists(self.log_path):
            os.makedirs(self.log_path)
        log_handler = logging.FileHandler(self.log_file_path)
        '''
        The log level written to the file defaults to debug.
        '''
        log_handler.setLevel(logging.DEBUG)
        log_handler.setFormatter(self._format)
        self.logger.addHandler(log_handler)

    def set_formatter_stream(self):
        console_handler = logging.StreamHandler()
        console_handler.setLevel(self._level)
        console_handler.setFormatter(self._format)
        self.logger.addHandler(console_handler)
