#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import sys
import time
import logging
import operator
import subprocess

from lmgrun import lmgwin
from lmgrun.lmglog import LMGLog
from lmgrun.cntrrun  import CntrRun
from lmgrun.inspector import Inspector
from lmgrun.validator import Validator
from lmgrun.argsparse import LMGArgsParse
from lmgrun.lmgenum import StatusCodeEnum
from lmgrun.baseinfo import BaseInfo, AppRunInfo

log = logging.getLogger(__name__)

def prepare_default_args_cold(uid):

    home_dir  = os.environ['HOME']
    args = ['--privileged', '-d', '--init', '--net', 'host', '--userns', 'host',  '--ipc', 'host', '-u', uid]
    defautlt_env = ['-e', 'LANG=zh_CN.UTF-8']
    default_volumes = ['-v', home_dir+':'+home_dir+':rw', '-v', '/:'+home_dir+'/portal:rw',
                       '-v', '/etc/passwd:/etc/passwd:ro', '-v', '/etc/shadow:/etc/shadow:ro', '-v', '/etc/group:/etc/group:ro', '-v', '/etc/gshadow:/etc/gshadow:ro']
    args += defautlt_env + default_volumes
    return args

def app_cold_start_prepare():
    aruninfo = AppRunInfo()

    default_args = prepare_default_args_cold(aruninfo.uid)
    run_args = default_args+aruninfo.env+aruninfo.devices+aruninfo.volumes
    log.debug("app cold start args:%s", run_args)
    return run_args

code_list=['0', '1', '137', '139', '255']
def app_container_status(ins):
    status = ins.get_status()
    if status == "running":
        log.debug("container status:%s" %(status))
        return StatusCodeEnum.APP_CONTAINER_RUNNING.code
    elif status == "exited":
        exited_code = ins.get_exited_code()
        if exited_code in code_list:
            log.debug("container status:%s now hot start" %(status))
            return StatusCodeEnum.APP_CONTAINER_COR_EXIT.code
        else:
            log.debug("container status:%s, exited code:%s" %(status, exited_code))
            return StatusCodeEnum.APP_CONTAINER_MIS_EXIT.code
    else:
        log.debug("container status:%s now cold start" %(status))
        return StatusCodeEnum.APP_CONTAINER_DNE.code

def app_check_cmd_is_same(ins_cmd):
    cmd = list(filter(None, (ins_cmd).strip("[]").split(" ")))
    log.debug("container cmd:%s, appcommand:%s", cmd, BaseInfo.cmd)
    return operator.eq(cmd, BaseInfo.cmd)

def app_start(crun, retcode, ins_cmd):
    if retcode == 10:
        if BaseInfo.parallel == True:
            crun.app_parallel_start()
            log.info ("%s already running, parallel:%d" %(BaseInfo.cntr_name, BaseInfo.parallel))
        else:
            log.info ("%s already running, not support open multiple copies" %(BaseInfo.cntr_name))
            msg = "{} already running" .format(BaseInfo.app_name)
            lmgwin.lmg_prompt("info", msg)
    elif retcode == 12:
        """
        When the container status is exited,
        start the container.
        """
        if app_check_cmd_is_same(ins_cmd) == True:
            crun.app_hot_start()
            log.info ("%s hot start" %(BaseInfo.cntr_name))
        else:
            crun.app_uninstall_container()
            run_args = app_cold_start_prepare()
            crun.app_cold_start(run_args)
            log.info ("%s already load, but the command is different, need to delete the container and call cold start" %(BaseInfo.cntr_name))
    elif retcode == 13:
        """
        Application error or container error, need to uninstall container, bug report
        """
        crun.app_uninstall_container()
        run_args = app_cold_start_prepare()
        crun.app_cold_start(run_args)
    else:
        log.debug("main will call app cold start")
        run_args = app_cold_start_prepare()
        crun.app_cold_start(run_args)

def lmglog_prepare(debug):
    if (debug == True):
       le = "debug"
    else:
       le = "info"
    LMGLog(level = le)

def feedback_prepare(fb):
    if (fb == True):
        feedback()

def lmg_app_run():
    cmdline = LMGArgsParse()
    binfo = BaseInfo(cmdline)
    lmglog_prepare(cmdline.args.debug)
    validator = Validator()
    crun  = CntrRun()
    code = validator.validate()

    #print("binfo.img_name:%s code:%d" %(binfo.img_name, code))
    if not code:
        run_args = app_cold_start_prepare()
        crun.app_cold_start(run_args)
    else:
        ins = Inspector()
        retcode = app_container_status(ins)
        app_start(crun, retcode, ins.get_command())

if __name__=='__main__':
    lmg_app_run()
