#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# @Desc: { 项目读取配置文件类模块 }

import os
import yaml
import logging

log = logging.getLogger(__name__)

class YamlLoadError(Exception):
    pass

class YamlDumpError(Exception):
    pass

class LMGYaml(object):
    def __init__(self, yamlfile, block=True):
        self.yamlfile = yamlfile
        if not block:
            self.data = self.yaml_load()

    def find_file_location(self, pkg_name):
        try:
            with open(self.yamlfile, 'r') as f:
                data = yaml.safe_load_all(f)
                for item in data:
                    if pkg_name == item['app']:
                        return item['apptag'][-1],  item['meta']
                return None, None
        except yaml.MarkedYAMLError as e:
            raise YamlLoadError('YAML Error: %s' % str(e))

    def yaml_load(self):
        try:
            with open(self.yamlfile,'r') as f:
                data = yaml.safe_load(f)
                #log.debug("YAML LOAD: %s", data)
        except yaml.MarkedYAMLError as e:
            log.error("YAML Error: %s", e)
            raise YamlLoadError("YAML Error: %s" % str(e))
        return data

    def get_value(self, key):
        if key in self.data:
            return self.data[key]
        else:
            log.error("The key:%s does not exist in the dictionary.", key)

if __name__=='__main__':
    ly = LMGYaml('/var/lib/lpkg/appindex.cfg')
    img_name, app_cfg_file = ly.find_file_location('weixin_2.1.4-1_1')
    #ay = LMGYaml("/opt/com.tencent.weixin_2.1.2_2.loong64.app/com.tencent.weixin_2.1.2_2.loong64.yaml", block = False)
    print('value:%s' %app_cfg_file)
