#!/usr/bin/env python
import os
import sys
import docker
import logging
import subprocess

from appinfo import Appinfo
from appcontainer import AppContainer

#INFO, DEBUG, WARNING
logging.basicConfig(level = logging.INFO, filename='./lmgapp_run.log', filemode='a', format = '%(asctime)s - %(name)s - %(levelname)s - %(message)s')
log = logging.getLogger(__name__)
def prepare_kwargs(uid):
    extra_hosts = {}

    kwargs = dict(
        #cap_add=['NET_ADMIN'],
        #cap_drop=[],
        #devices=devices,
        #dns=config.get('dns'),
        #dns_search=[],
        #extra_hosts=extra_hosts,
        network_mode="host",
        ipc_mode="host",
        userns_mode="host",
        user=uid,
        #ports=port_bindings,
        privileged=True,
        #publish_all_ports=False,
        #restart_policy={'Name': 'no'},
        stdin_open=False,
        stdout=False,
        #volumes=volumes,
        remove=False
    )
    return kwargs

def switch_to_case_device(argument):
    switcher = {
        "Sound" : ["/dev/snd:/dev/snd", "/dev/dri:/dev/dri"],
        "Camera": ["/dev/video0:/dev/video0"],
    }

    return switcher.get(argument, "nothing")

def switch_to_case_volume(argument):
    switcher = {
        "X11"   : {'/tmp/.X11-unix':{'bind':'/tmp/.X11-unix', 'mode': 'ro'}},
        "Cups"  : {'/var/run/cups':{'bind':'/var/run/cups', 'mode': 'ro'}},
    }

    return switcher.get(argument, "nothing")


def switch_to_case_env(argument):
    #switcher = {
    #    "Sound" : "--device /dev/snd --device /dev/dri -e PULSE_SERVER=unix:"+os.getenv('XDG_RUNTIME_DIR')+"/pulse/native ",
    #    "X11"   : "-v /tmp/.X11-unix:/tmp/.X11-unix -e DISPLAY="+os.getenv('DISPLAY')+ " ",
    #    "Cups"  : "-v /tmp/client.conf:/etc/cups/client.conf:ro ",
    #    "Creama": "-v /dev/video0:/dev/video0",
    #}

    #-e CLUTTER_IM_MODULE=xim -e GTK_IM_MODULE=xim -e QT4_IM_MODULE=xim -e XMODIFIERS=@im=fcitx -e LANG=C.UTF-8  \

    switcher = {
        "Sound"    : ["PULSE_SERVER=unix:"+str(os.getenv('XDG_RUNTIME_DIR'))+"/pulse/native"],
        "X11"      : ["DISPLAY="+os.getenv('DISPLAY')],
        "IM-input" : ["CLUTTER_IM_MODULE=xim", "GTK_IM_MODULE=xim", "QT4_IM_MODULE=xim", "XMODIFIERS=@im=fcitx", "LANG=C.UTF-8"],
    }

    return switcher.get(argument, "nothing")

def prepare_devices(permission):
    devices=[]
    for it in permission:
        arg=switch_to_case_device(it)
        if arg == 'nothing' :
            log.debug("no permission:%s in device directory" %(it))  # Prints String are equal
        else :
            devices += arg

    return devices

def prepare_volumes(permission, home):
    volumes = {home:{'bind': home, 'mode': 'rw'}, # Shared host home directory
                '/':{'bind': home+'/portal', 'mode': 'rw'}, # Shared host root directory
               '/etc/passwd':{'bind':'/etc/passwd', 'mode': 'ro'}, #Shard host passwd
               '/etc/shadow':{'bind':'/etc/shadow', 'mode': 'ro'}, #Shard host shadow
               '/etc/group':{'bind':'/etc/group', 'mode': 'ro'}, # Shard host group
               '/etc/gshadow':{'bind':'/etc/gshadow', 'mode': 'ro'} #shard host gshasow
               }
    for it in permission:
        arg=switch_to_case_volume(it)
        if arg == 'nothing' :
            log.debug("no permission in directory")  # Prints String are equal
        else :
            volumes.update(arg)

    return volumes

def prepare_environment(permission):
    """
    Prepare environment variables for a app container.
    """
    # Make a copy so that we do not alter the original, which only contains
    # user-specified environment variables.
    #
    # Start with the app environment variables, then apply service-specific
    # variables on top of that.
    #env = dict()
    env = []
    #env['DISPLAY'] = os.getenv('DISPLAY')
    #env['PULSE_SERVER'] = "unix:"+str(os.getenv('XDG_RUNTIME_DIR'))+"/pulse/native"
    for it in permission:
        arg=switch_to_case_env(it)
        if arg == 'nothing' :
            log.debug("no permission:%s in env directory" %(it))  # Prints String are equal
        else :
            env += arg

    return env

def start_container(appinfo):
    """
    Start app.
    """

    kwargs = prepare_kwargs(appinfo.uid)

    environment = prepare_environment(appinfo.permission)
    devices =  prepare_devices(appinfo.permission)
    volumes = prepare_volumes(appinfo.permission, appinfo.home)

    log.debug("env:%s, devices:%s volumes:%s" %(environment, devices, volumes))

    command=appinfo.command

    appcontainer = AppContainer(appinfo.get_image_name(), appinfo.get_container_name())
    appcontainer.app_container_start(True, command, environment, devices, volumes, kwargs)

lmgcmd="lmgconf_get"
def lmgconf_tool_check():
    lmgconftoolexist=0
    lbin=lmgcmd
    for cmdpath in os.environ['PATH'].split(':'):
        if os.path.isdir(cmdpath) and lbin in os.listdir(cmdpath):
            lmgconftoolexist=1
    if not lmgconftoolexist:
        #TODO:Switch to pop-up notification
        log.info ("I require %s but it's not installed.  Aborting." %(lbin))
        exit(1)

def appinfo_key_value(yamlfile, key):
    cmd=lmgcmd+' '+yamlfile+' '+key
    try:
        out_bytes = subprocess.check_output(cmd, shell=True)
    except subprocess.CalledProcessError as e:
        out_bytes = e.output       # Output generated before error
        code      = e.returncode   # Return code
        exit(1)
    #out_text = out_bytes.decode('utf-8')
    return out_bytes.strip();

def read_info_from_yaml(appinfo):
    appinfo.uid = os.getuid()
    appinfo.home = os.environ['HOME']
    appinfo.name = appinfo_key_value(appinfo.yamlfile, "appname")
    appinfo.version = appinfo_key_value(appinfo.yamlfile, "appversion")
    permission = appinfo_key_value(appinfo.yamlfile, "permission")
    appinfo.permission = permission.split()
    appinfo.command = appinfo_key_value(appinfo.yamlfile, "run")

def appcontainer_status(appinfo):
    appcontainer = AppContainer(appinfo.get_image_name(), appinfo.get_container_name())
    status = appcontainer.get_status()
    if status == "running":
        return 0
    elif status == "exited":
        log.info("container status:%s now rm it" %(status))
        appcontainer.app_container_rm()
        return 1
    else:
        return 2

if __name__=='__main__':
    try:
        if len(sys.argv) != 2:
            log.info ("USAGE: %s yamlfile" %(sys.argv[0]))
            exit (1)
    except Exception as e:
        raise e

    yamlfile = sys.argv[1]
    lmgconf_tool_check()
    appinfo = Appinfo()
    appinfo.yamlfile = yamlfile
    read_info_from_yaml(appinfo)
    log.debug("name:%s version:%s permission:%s uid:%s conid:%s" %(appinfo.name, appinfo.version, appinfo.permission, appinfo.uid, appinfo.get_container_name()))
    status = appcontainer_status(appinfo)
    if status == 0:
        log.info ("%s already running" %(appinfo.get_container_name()))
    else:
        start_container(appinfo)
