# -*- coding: utf-8 -*-
import sys
import subprocess
import logging
from time import sleep
from ..utils.execute import execute
from ..utils.exceptions import DockerLoadError, DockerLsError, DockerTagError, DockerInspectError, DockerRmiError, DockerKillError, AppRunError, DockerAppImage

log = logging.getLogger(__name__)

def load_image(imagename, lmgtype):
    if lmgtype == 'BASERUNTIME':
        result = execute(['loongker', 'load', '-i', imagename], stdout=sys.stdout)

    elif lmgtype == 'APPLICATIONS':
        result = execute(['loongker', 'loadapp', '-i', imagename], stdout=sys.stdout)

    elif lmgtype == 'UPBASERUNTIME':
        result = execute(['loongker', 'loadupgrade', '-i', imagename], stdout=sys.stdout)

    else :
        raise PackError('lmgtype error: %s ' %lmgtype)

    if not result.succeeded:
        raise DockerLoadError(result.__dict__)

def rm_applayer(applayer):
    runcmd = ['loongker', 'rma', applayer]
    result = execute(runcmd)
    #if not result.succeeded:
    #    raise DockerRmiError(result.__dict__)

def force_rm_applayer(applayer):
    runcmd = ['loongker', 'rma', applayer]
    result = execute(runcmd)

class AppImage(object):
    def __init__(self, repository):
        self.repo = repository
        self.tags = self.get_image_tag()

    def get_image_tag(self):
        runcmd = ['loongker', 'image', 'ls', '--format', '{{.Tag}}',
                self.repo]
        result = execute(runcmd)
        if not result.succeeded:
            raise DockerLsError(result.__dict__)

        #if not result.out:
        #    raise DockerAppImage('Image generation failed.') 
        return result.out.split()

    def set_image_name(self):
        target_images = []
        for source_tag in self.tags:
            runcmd = ['loongker', 'image', 'inspect', '--format', '{{.RepoTags}}', source_tag]
            result = execute(runcmd)
            if not result.succeeded:
                raise DockerInspectError(result.__dict__)

            target_tag = result.out.strip('[]').split(':', 1)[0]
            source_image = self.repo + ':' + source_tag
            target_image = self.repo.rsplit('.',1)[0] + ':' + target_tag
            runcmd = ['loongker', 'tag', source_image, target_image]
            result = execute(runcmd)
            if not result.succeeded:
                raise DockerTagError(result.__dict__)

            runcmd = ['loongker', 'rmi', source_image]
            result = execute(runcmd)
            target_images.append(target_image)
            if not result.succeeded:
                raise DockerRmiError(result.__dict__)
        return target_images

class AppContainer(object):
    """
    Class for accessing information about a app's container.
    """
    def __init__(self, image_name):
        self.image_name = image_name
        self.container_list = self.get_container_name()

    def get_container_name(self):
        runcmd = ['loongker', 'ps', '-a', '--format', 'table {{.Names}}', '--filter',
            'ancestor=%s' %self.image_name]
        result = execute(runcmd)
        res = result.out.split()
        res.remove("NAMES")
        return res

    def get_container_status(self):
        """
        Return the status of the container (running, exited, paused).
        Returns "missing" if the app does not exist.
        """
        res = []
        for container_name in self.container_list:
            runcmd = ['loongker', 'inspect', '--format', '{{.State.Status}}', container_name]
            result = execute(runcmd)
            res.append(result.out.strip())
            if not result.succeeded:
                raise DockerInspectError(result.__dict__)
        return res

    def kill_container(self):
        sleep(2)

        for container_name in self.container_list:
            """loongker kill <container name>"""
            runcmd=['loongker', 'kill']
            runcmd.append(container_name)
            result = execute(runcmd)
            if not result.succeeded:
                raise DockerKillError(result.__dict__)

    def rm_container(self):
        for container_name in self.container_list:
            runcmd = ['loongker', 'rm', '-f']
            runcmd.append(container_name)
            result = execute(runcmd)
            if not result.succeeded:
                raise DockerRmiError(result.__dict__)

    def rm_image(self):
        runcmd = ['loongker', 'image', 'rm', '-f', self.image_name]
        result = execute(runcmd)
        if not result.succeeded:
            raise DockerRmiError(result.__dict__)

    def app_uninstall_container(self):
        container_status = self.get_container_status()
        for status in container_status:
            if status == 'running':
                raise AppRunError(self.image_name)
        self.rm_container()
        self.rm_image()

    def app_force_uninstall_container(self):
        #TODO: need bug report
        try:
            self.rm_container()
            self.rm_image()
            log.debug("Successfully rm container: %s, res: %s\n" %(self.image_name, res))
        except Exception as e:
            log.debug("Failed to rm container:%s, return:%s\n" %(self.image_name, e))

#     appc = AppContainer('debian')
#     appc.get_status()
#     appc.app_uninstall_container()
