import os
import yaml
import shutil
import logging
from ..utils.execute import execute
from .appc import AppContainer, AppImage, rm_applayer, force_rm_applayer
from ..utils.exceptions import RmFileError

log = logging.getLogger(__name__)

class YamlFormat(object):
    def __init__(self, yamlfile):
        self.yamlfile = yamlfile

    def yaml_add(self, document):
        try:
            with open(self.yamlfile, 'a+') as f:
                yaml.safe_dump_all(document, f, explicit_start = True, explicit_end = True)
        except yaml.YAMLError:
            raise yaml.YAMLError("YAML data dump err")

    def yaml_isexist(self, key, value):
        try:
            with open(self.yamlfile, 'r') as f:
                data = yaml.safe_load_all(f)
                for item in data:
                    if self.item_isexist(item, key, value):
                        return True
                return False
        except yaml.MarkedYAMLError as e:
            raise yaml.YAMLError('YAML Error: %s' %str(e))

    def item_isexist(self, item, key, value):
        for yamlkey in item.keys():
            if key == yamlkey and value == item[yamlkey]:
                return True
        return False

    def yaml_dele(self, key, value):
        try:
            with open(self.yamlfile, 'r') as f1, open('%s.bak' % self.yamlfile, 'w+') as f2:
                data = yaml.safe_load_all(f1)
                for item in data:
                    if not self.item_isexist(item, key, value):
                           yaml.safe_dump_all([item], f2, explicit_start = True, explicit_end = True)
            os.remove(self.yamlfile)
            os.rename('%s.bak' %self.yamlfile, self.yamlfile)

        except yaml.YAMLError:
            raise yaml.YAMLError("YAML data dump err")


class Lmgstatus(YamlFormat):
    def yaml_print(self):
        try:
            with open(self.yamlfile, 'r') as f:
                data = yaml.safe_load_all(f)
                # fmt_header = '{:<23} {:<23} {:<6} {:<8}'
                # print(fmt_header.format('名称', '版本', '体系结构', '描述'))
                # for item in data:
                #     fmt = '{:<25} {:<25} {:<10} {:<10}'
                #     print(fmt.format(item['appname'], item['appversion'], item['arch'], item['description'].split(os.linesep, 1)[0]))

                fmt_header = '{:<23} {:<23} {:<6} '
                print(fmt_header.format('名称', '版本', '体系结构'))
                for item in data:
                    fmt = '{:<25} {:<25} {:<10}'
                    print(fmt.format(item['appname'], item['appversion'], item['arch']))
        except yaml.YAMLError:
            raise yaml.YAMLError("YAML load data err")

class Appindex(YamlFormat):
    def yaml_print(self, key, value):
        try:
            with open(self.yamlfile, 'r') as f:
                data = yaml.safe_load_all(f)
                for item in data:
                    if self.item_isexist(item, key, value):
                        for apptag in item['apptag']:
                            print(apptag)
                        break

        except yaml.YAMLError:
            raise yaml.YAMLError("YAML data dump err")

    def yaml_dele(self, key, value):
        try:
            with open(self.yamlfile, 'r') as f1, open('%s.bak' % self.yamlfile, 'w+') as f2:
                data = yaml.safe_load_all(f1)
                for item in data:
                    if not self.item_isexist(item, key, value):
                       yaml.safe_dump_all([item], f2, explicit_start = True, explicit_end = True)
                    else:
                        for apptag in item['apptag']:
                            appcontainer = AppContainer(apptag)
                            appcontainer.app_uninstall_container()
                        if item['lmgtype'] == 'APPLICATIONS':
                            rm_applayer(item['applayer'])
            os.remove(self.yamlfile)
            os.rename('%s.bak' %self.yamlfile, self.yamlfile)

        except yaml.YAMLError:
            raise yaml.YAMLError("YAML data dump err")

    def force_yaml_dele(self, key, value):
        try:
            with open(self.yamlfile, 'r') as f1, open('%s.bak' % self.yamlfile, 'w+') as f2:
                data = yaml.safe_load_all(f1)
                for item in data:
                    if not self.item_isexist(item, key, value):
                       yaml.safe_dump_all([item], f2, explicit_start = True, explicit_end = True)
                    else:
                        for apptag in item['apptag']:
                            appcontainer = AppContainer(apptag)
                            appcontainer.app_force_uninstall_container()
                        if item['lmgtype'] == 'APPLICATIONS':
                            force_rm_applayer(item['applayer'])
            os.remove(self.yamlfile)
            os.rename('%s.bak' %self.yamlfile, self.yamlfile)

        except yaml.YAMLError:
            raise yaml.YAMLError("YAML data dump err")


    def yaml_up_apptag(self):
        try:
            with open(self.yamlfile, 'r') as f1, open('%s.bak' % self.yamlfile, 'w+') as f2:
                data = yaml.safe_load_all(f1)
                for item in data:
                    if 'applayer' in item:
                        appimage = AppImage(item['applayer'])
                        apptag = appimage.set_image_name()[:]
                        item['apptag'].extend(apptag)
                    yaml.safe_dump_all([item], f2, explicit_start= True, explicit_end = True)
            os.remove(self.yamlfile)
            os.rename('%s.bak' % self.yamlfile, self.yamlfile)

        except yaml.YAMLError:
            raise yaml.YAMLError("YAML data dump err")

class Lmgmeta(object):
    def __init__(self, yamlfile):
        self.yamlfile = yamlfile
        self.data = {}

    def yaml_load(self):
        try:
            with open(self.yamlfile, 'r') as f:
                self.data = yaml.safe_load(f)
        except yaml.MarkedYAMLError as e:
            raise yaml.YAMLError("YAML Error: %s"  % e)

    def get_value(self, key):
        if key in self.data:
            return self.data[key]
        else:
            raise KeyError("The key : %s does not exist in the dictionary." % key)

    def yaml_print(self):
        print(f'Appname: {self.get_value("appname")}')
        print(f'Version: {self.get_value("appversion")}')
        print(f'Arch: {self.get_value("arch")}')
        print(f'Depends: {self.get_value("dependence")}')
        print(f'Description: {self.get_value("description")}')

class Repofile(Lmgmeta):
    def yaml_search(self, app, arch):
        if os.path.isfile(self.yamlfile):
            self.yaml_load()
            for appone in self.get_value(arch):
                if appone == app:
                    return True
            return False
        else:
            log.warning('Could not find '+ self.yamlfile +', which is assumed to have no files in the system.')


class Applist(Lmgmeta):
    def yaml_dump(self, document):
        try:
            with open(self.yamlfile, 'w+') as f:
                yaml.safe_dump(document, f)
        except yaml.YAMLError:
            raise yaml.YAMLError("YAML data dump err")

    def yaml_print(self):
        if os.path.isfile(self.yamlfile):
            self.yaml_load()

            for appfile in self.get_value('files'):
                print(appfile)

            for appdir in self.get_value('dirs'):
                print(appdir)
        else:
            log.warning('Could not find '+ self.yamlfile +', which is assumed to have no files currently installed in the system.')


    def rmfile(self):
        if os.path.isfile(self.yamlfile):
            self.yaml_load()

            for appdir in self.get_value('dirs'):
                if os.path.isdir(appdir):
                    shutil.rmtree(appdir)

            for appfile in self.get_value('files'):
                if os.path.isfile(appfile):
                    os.remove(appfile)

            os.remove(self.yamlfile)
        else:
            log.warning('Could not find '+ self.yamlfile +', which is assumed to have no files currently installed in the system.')
