#!/usr/bin/env python3
#-*- coding: utf-8 -*-
import os
import sys
import pwd
import fcntl
import logging
import getpass
from  ..utils.parser import Parser
from  ..utils.logger import log_init
from  ..utils.execute import execute, get_lpkg_path
from  .lmgmanager import LmgFactory, LmgOption, lpkg_init

log = logging.getLogger(__name__)

class StatLock(object):
    def __init__(self, filename):
        self.fd = open(filename, 'w+')

    def lock(self):
        fcntl.flock(self.fd, fcntl.LOCK_EX | fcntl.LOCK_NB)

    def unlock(self):
        fcntl.flock(self.fd, fcntl.LOCK_UN)

def root_check():
    if getpass.getuser() != 'root':
        log.error('Run permission error, please run with sudo ！')
        sys.exit()

def args_init(cmdline):
    if not os.path.isdir(cmdline.path):
        os.makedirs(cmdline.path)

    lmgfactory = LmgFactory()
    lmgoption = LmgOption(lmgfactory)

    if cmdline.list:
        lmgoption.CreateLmgList()
    elif cmdline.app_lf:
        lmgoption.CreateLmgListFile(cmdline.app_lf)
    elif cmdline.lmgfield:
        lmgoption.CreateLmgField(cmdline.lmgfield)
    else:
        root_check()
        statlock = StatLock('/var/lib/lpkg/lock')
        try:
            statlock.lock()
            #for pathname in cmdline.LmgFile:
            #    lmgoption.CreateLmgInst(pathname, cmdline.path, cmdline.udir)

            for pathname in cmdline.lmgfile:
                lmgoption.CreateLmgInst(pathname, cmdline.path, cmdline.udir)

            for app in cmdline.app:
                lmgoption.CreateLmgUnInst(app)

            statlock.unlock()
        except BlockingIOError:
            log.warning('The program does not support parallelism')
            sys.exit(1)

def lmg_init():
    parser = Parser()
    cmdline = parser.parse(sys.argv)

    log_init(get_lpkg_path(), 'lpkg.log', cmdline.debug)
    args_init(cmdline)

if __name__ == '__main__':
    lmg_init()




