#!/usr/bin/python3
# -*- coding: utf-8 -*-
# @Desc: { 双击安装窗口模块 }

import os
import sys
import tkinter as tk
from tkinter import ttk
from tkinter.messagebox import *
import subprocess
import threading

class ProcessResult(object):
    return_code = None
    out = ''
    err = ''

    def __init__(self, command):
        self.command = command

    @property
    def succeeded(self):
        if self.return_code is None:
            return None
        return self.return_code == 0

def execute(cmd, env=None, cwd=None, stdin='', stdout=subprocess.PIPE):
    result = ProcessResult(command=cmd)
    #log.debug('Running command: "{0}"'.format(cmd))
    process = subprocess.Popen(
        cmd,
        env=env,
        cwd=cwd,
        stdin=subprocess.PIPE,
        stdout=stdout,
        stderr=subprocess.PIPE,
        close_fds=True
    )

    (stdout, stderr) = process.communicate(str.encode(stdin))

    result.out = stdout.decode('utf-8') if stdout else ''
    result.err = stderr.decode('utf-8') if stderr else ''
    result.return_code = process.returncode
    #log.debug('Finished running of: {0}'.format(result.__dict__))
    return result

def lmg_prompt(level, msg):
    #window = tk.Tk()
    #window.withdraw()  # 退出默认 tk 窗口

    if (level == 'warning'):
        result = showwarning('警告', msg)
        #print(f'警告: {result}')
    elif (level == 'error'):
        result = showerror('错误', msg)
        #print(f'错误: {result}')
    else:
        result = showinfo('提示', msg)
        #print(f'提示: {result}')

def init_ui(filename):
    # root window
    root = tk.Tk()
    # windown center
    width = 360
    height =120
    screenwidth = root.winfo_screenwidth()
    screenheight = root.winfo_screenheight()
    size_geo = '%dx%d+%d+%d' % (width, height, (screenwidth-width)/2, (screenheight-height)/2)
    root.geometry(size_geo)
    root.title('Install Package')
    root.grid()

    # progressbar
    pb = ttk.Progressbar(
        root,
        orient='horizontal',
        mode='indeterminate',
        length=310
    )
    # place the progressbar
    pb.grid(column=0, row=0, columnspan=2, padx=20, pady=20)

    # start button
    start_button = ttk.Button(
        root,
        text='安装'
    )
    start_button.config(command=lambda:[pb.start(), bu_start(root, filename,start_button)])
    start_button.grid(column=0, row=1, padx=10, pady=10, sticky=tk.E)

    # stop button
    stop_button = ttk.Button(
        root,
        text='取消',
        #command=pb.stop
        command=lambda:[root.destroy(), bu_stop()] 
    )
    stop_button.grid(column=1, row=1, padx=10, pady=10, sticky=tk.W)

    root.mainloop()

def bu_switch_state(event):
    #if event.widget['state'] == 'normal':
    event.widget['state'] = 'disabled'

def run_lpkg(root, filename):
    os.environ['LMGDISPLAY'] = 'True'
    runcmd = ['/usr/local/bin/lpkg', '-i', filename]
    result = execute(runcmd, cwd = os.path.dirname(filename))
    
    root.withdraw()
    if result.succeeded:
        lmg_prompt('info', '软件安装成功！')
    else:
        lmg_prompt('error', '软件安装失败！') 
    root.destroy()

def bu_start(root, filename, button):
    button.config(state=tk.DISABLED) 
    T = threading.Thread(target=run_lpkg, args=(root, filename), daemon=True) 
    T.start()

def bu_stop():
    sys.exit()

if __name__ == '__main__':
    filename = sys.argv[1]
    if filename.startswith('file://'):
        filename = filename[7:]
    init_ui(filename)
