import re
import os 
import pwd
import shutil
import logging
import fnmatch
import tempfile
from ..utils.execute import execute, get_lpkg_path
from ..utils.exceptions import DecompressError, PackError

log = logging.getLogger(__name__)

class UnPackage(object):
    def __init__(self, pathname, udir):
        self.pathname = pathname
        self.udir = udir
    
    #1. detect file existence
    #2. detect file suffix
    def check_file(self, filename, filesuffix):
        if not os.path.isfile(filename):
            raise PackError(f'File does not exist: {filename}')

        if not filename.endswith(filesuffix):
            raise PackError('Wrong Lmg Package !!!')

    def check_extract_environment(self, arfile, directory):
        #if not os.path.isfile('/usr/local/lmg/bin/zstd'):
        #    log.error('Missing dpendency zstd.')

        if os.statvfs(directory).f_bavail < 1024:
            log.error('Home dir storage space is too small.\n \
                        Please use -t to select unzip dir!')

        if os.path.getsize(arfile) < 1024:
            log.error('Lmg package is broken.')

    def extract(self, arfile, directory):
        lmg_env = {'PATH': os.environ['PATH'] + os.pathsep +
                '/usr/local/lmg/bin'}
        result = execute(
                ['tar', '-xf', arfile, '-C', directory], 
                env=lmg_env)

        if not result.succeeded:
            self.check_extract_environment(arfile, directory)
            shutil.rmtree(directory)
            raise DecompressError(arfile)
        return directory

    def do_unpack(self):
        self.check_file(self.pathname, '.lmg')
        if not os.path.isdir(self.udir):
            os.makedirs(self.udir)
            os.chown(self.udir, 
                    pwd.getpwnam(os.getlogin()).pw_uid,
                    pwd.getpwnam(os.getlogin()).pw_gid)
        elif os.listdir(self.udir):
            raise DecompressError('Please specify an empty decompression directory !')
        
        self.extract(self.pathname, self.udir)
        self.un_data(self.udir)
        self.un_lcf(self.udir)

    def un_data(self, directory):
        for filename in os.listdir(directory):
            if filename.endswith('.app'):
                directory = directory + '/' + filename
                arfile = directory + '/' + 'data.tar.gz'
                break

        self.check_file(arfile, '.tar.gz')
        self.extract(arfile, directory)
        os.remove(arfile)

    def un_lcf(self, directory):
        for filename in os.listdir(directory):
            if filename.endswith('.app'):
                directory = directory + '/' + filename
                arfile = directory + '/' + 'lcf.tar.gz'
                break

        self.check_file(arfile, '.tar.gz')
        self.extract(arfile, directory)
        os.remove(arfile)

    def do_field(self):
        self.check_file(self.pathname, '.lmg')
        if not os.path.isdir(self.udir):
            os.makedirs(self.udir)
            os.chown(self.udir, 
                    pwd.getpwnam(os.getlogin()).pw_uid,
                    pwd.getpwnam(os.getlogin()).pw_gid)
        elif os.listdir(self.udir):
            raise DecompressError('Please specify an empty decompression directory !')

        self.extract(self.pathname, self.udir)
        self.un_lcf(self.udir)

class Lmgformat(object):
    def __init__(self, udir):
        for filename in os.listdir(udir):
            if filename.endswith('.app'):
                path = udir + '/' +filename
        self.path = path    
        self.dict = {}
        self.desktop = {}

    def load_desktop(self):
        basename = os.path.basename(self.path)
        dirname = self.path + '/Data/AppData'

        icon = []
        desktop = [filename for filename in os.listdir(dirname)
                    if filename.endswith('.desktop')]

        for filename in desktop:        
            with open(dirname + '/' + filename, 'r') as f:
                for line in f:
                    if re.match('Icon=', line):
                        *_, name = re.split('=|/', line)
                        icon.append(name.strip())
                        
        for key,value in zip(desktop,icon):
            for filename in os.listdir(dirname):
                if re.match(value + '(?!.desktop)', filename):
                    value = filename
                    break
            self.desktop[basename+'/Data/AppData/'+key] = basename+'/Data/AppData/'+value

                
    def load_dict(self):
        basename = os.path.basename(self.path)
        file_prefix_name = basename.rsplit('.', 1)[0]


        def get_image_name():
            for filename in os.listdir(self.path):
                if fnmatch.fnmatch(filename, '*.tar'):
                    return basename + '/' + filename
            return None

        self.dict['meta'] = os.path.join(basename, file_prefix_name + '.yaml')
        self.dict['image'] = get_image_name()

    def get_desktop_dict(self):
        return self.desktop

    def get_file_path(self, key):
        if key in self.dict:
           return self.dict[key]
        else:
            log.error("The key:%s does not exist in the dictionary.", key)

    def check_file_exist(self):
        for value in self.dict.values():
            if value and not os.path.isfile(os.path.dirname(self.path)+'/'+value):
                raise PackError('broken package')
       #desktop ---->  icon
       #1. icon and desktop exist: check  
       #   load_desktop will find all
       #2. desktop exist, icon miss: check
       #   load_desktop will find desktop
       #3. desktop miss, icon exist: check
       #   load_desktop not find file

        if not self.desktop and len(os.listdir(self.path + '/Data/AppData')) > 1:
            raise PackError('broken package')

        for key,value in self.desktop.items():
            if (
            (key and not os.path.isfile(os.path.dirname(self.path)+'/'+key)) or 
            (value and not os.path.isfile(os.path.dirname(self.path)+'/'+value)) 
                ):
                raise PackError('broken package')



                
