import logging
from .lmgwin import show_message

log = logging.getLogger(__name__)

class LpkgError(Exception):
    def __init__(self, message):
        log.error(message)
        show_message('error', message)

class DependError(Exception):
    def __init__(self, message):
        info = 'Dependence error: ({})'.format(message)
        log.error(info)
        show_message('error', info)
        
class PackError(Exception):
    def __init__(self, message):
        info = 'Package error: ({})'.format(message)
        log.error(info)
        show_message('error', info)

class ZstdError(LpkgError):
    def __init__(self, message):
        info = 'Missing dependency zstd: ({})'.format(message)
        log.error(info)
        show_message('error', info)

class SpaceError(LpkgError):
    def __init__(self, message):
        info = 'Storage space error'.format(message)
        log.error(info)
        show_message('error', info)

class AppRunError(LpkgError):
    def __init__(self, message):
        info = 'Container is running: ({})'.format(message)
        log.error(info)
        show_message('error', info)

class RuncmdError(LpkgError):
    def __init__(self, command):
        message = 'run cmd error: ({})'.format(command)
        log.error(message)
        show_message('error', message)
    
class RmFileError(RuncmdError):
    def __init__(self, command):
        message = 'remove file error: ({})'.format(command)
        log.error(message)
        show_message('error', message)

class DecompressError(RuncmdError):
    def __init__(self, command):
        message = 'unzip lmg error: ({})'.format(command)
        log.error(message)
        show_message('error', message)
        
class DockerLoadError(RuncmdError):
    def __init__(self, command):
        message = 'loongker load error: ({})'.format(command)
        log.error(message)
        show_message('error', message)

class DockerTagError(RuncmdError):
    def __init__(self, command):
        message = 'loongker tag error: ({})'.format(command)
        log.error(message)
        show_message('error', message)

class DockerAppImage(LpkgError):
    def __init__(self, message):
        info = 'Image error: ({})'.format(message)
        log.error(info)
        show_message('error', info)

class DockerLsError(RuncmdError):
    def __init__(self, command):
        message = 'loongker image ls error: ({})'.format(command)
        log.error(message)
        show_message('error', message)

class DockerInspectError(RuncmdError):
    def __init__(self, command):
        message = 'loongker image inspect error: ({})'.format(command)
        log.error(message)
        show_message('error', message)

class DockerRmiError(RuncmdError):
    def __init__(self, command):
        message = 'loongker image rmi error: ({})'.format(command)
        log.error(message)
        show_message('error', message)

class DockerKillError(RuncmdError):
    def __init__(self, command):
        message = 'loongker kill container error: ({})'.format(command)
        log.error(message)       
        show_message('error', message)
