#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import os
import pwd
import sys
import logging
import subprocess

log = logging.getLogger(__name__)
def get_lpkg_path():
    return os.path.join(pwd.getpwnam(os.getlogin()).pw_dir, '.lmg/lpkg')

def get_depend_path():
    return os.path.join(pwd.getpwnam(os.getlogin()).pw_dir, '.lmg/lpkg/depend')

def euid_to_login():
    os.setegid(pwd.getpwnam(os.getlogin()).pw_gid)
    os.seteuid(pwd.getpwnam(os.getlogin()).pw_uid)

def euid_to_root():
    os.seteuid(0)
    os.setegid(0)

class ProcessResult(object):
    return_code = None
    out = ''
    err = ''

    def __init__(self, command):
        self.command = command

    @property
    def succeeded(self):
        if self.return_code is None:
            return None
        return self.return_code == 0

    #@property
    #def succeeded(self):
    #    if self.err :
    #        return 0
    #    else:
    #        return 1

def execute(cmd, env=None,stdin='', stdout=subprocess.PIPE):
    result = ProcessResult(command=cmd)
    log.debug('Running command: "{0}"'.format(cmd))
    process = subprocess.Popen(
        cmd,
        env=env,
        stdin=subprocess.PIPE,
        stdout=stdout,
        stderr=subprocess.PIPE,
        close_fds=True
    )

    (stdout, stderr) = process.communicate(str.encode(stdin))

    result.out = stdout.decode('utf-8') if stdout else ''
    result.err = stderr.decode('utf-8') if stderr else ''
    result.return_code = process.returncode
    log.debug('Finished running of: {0}'.format(result.__dict__))
    return result

#if __name__ == '__main__':
#
#    runcmd = ['caja', '-q']
#    result = execute(runcmd)
#    print (result.out)
#    print (result.err)
