import os
import sys
import pwd 
import time
import logging
from colorama import Fore,Style

LOG_ENABLED = True
LOG_LEVEL = 'DEBUG'
LOG_FORMAT = '%(asctime)s - %(levelname)s - %(name)s - %(funcName)s - %(message)s'
LOG_FORMAT_CONSOLE = '%(module)s - %(levelname)s - %(message)s'
class CustomFormatter(logging.Formatter):
    def __init__(self, fmt):
        self.fmt = fmt
        self.FORMATS = {
            logging.DEBUG: Fore.WHITE + self.fmt + Style.RESET_ALL,
            logging.INFO: Fore.GREEN + self.fmt + Style.RESET_ALL,
            logging.WARNING: Fore.YELLOW + self.fmt + Style.RESET_ALL,
            logging.ERROR: Fore.RED + self.fmt + Style.RESET_ALL,
            logging.CRITICAL: Fore.RED + self.fmt + Style.RESET_ALL
        }

    def format(self, record):
        log_fmt = self.FORMATS.get(record.levelno)
        formatter = logging.Formatter(log_fmt)
        return formatter.format(record)

class Log(object):
    def __init__(self):
        self.logger = logging.getLogger()
        self.logger.setLevel(LOG_LEVEL)
        self.formatter = logging.Formatter(LOG_FORMAT)

    def log_to_console(self, log_level = LOG_LEVEL):
        ''' log output to terminal
        :param self: 
        :param log_level: log output level in terminal
        '''
        if LOG_ENABLED:
            stream_handler = logging.StreamHandler(sys.stdout)
            stream_handler.setLevel(log_level)
            #formatter = logging.Formatter(LOG_FORMAT_CONSOLE)
            formatter = CustomFormatter(LOG_FORMAT_CONSOLE)
            stream_handler.setFormatter(formatter)
            self.logger.addHandler(stream_handler)

    def recursive_chown(self, path, uid, gid):
        for root, dirs, files in os.walk(path):
            os.chown(root, uid, gid)
            for name in dirs:
                os.chown(os.path.join(root, name), uid, gid)
            for name in files:
                os.chown(os.path.join(root, name), uid, gid)

    def log_to_file(self, log_path, log_file, log_level = LOG_LEVEL):
        '''log output to file
        :param self:
        :param log_file: logging file name
        :param log_level: log output level in file
        '''
        if LOG_ENABLED:
            if not os.path.isdir(log_path):
                os.makedirs(log_path)
                os.mknod(os.path.join(log_path, log_file), 0o644)
                self.recursive_chown(os.path.dirname(log_path),
                         pwd.getpwnam(os.getlogin()).pw_uid,
                         pwd.getpwnam(os.getlogin()).pw_gid)
            elif not os.path.isfile(os.path.join(log_path, log_file)):
                os.mknod(os.path.join(log_path, log_file), 0o644)
                self.recursive_chown(os.path.dirname(log_path),
                         pwd.getpwnam(os.getlogin()).pw_uid,
                         pwd.getpwnam(os.getlogin()).pw_gid)

            #log_file to do
            file_handler = logging.FileHandler(os.path.join(log_path,log_file), encoding='utf-8')
            file_handler.setLevel(log_level)
            file_handler.setFormatter(self.formatter)
            self.logger.addHandler(file_handler)

def log_init(log_path, log_file, debug = False):
    '''logging initialization: set logging level and log output to terminal
    :param debug: debug switch
    :returns log: log object
    '''
    if debug:
        log_level = 'DEBUG'
    else:
        log_level = 'INFO'

    log = Log()
    log.log_to_console(log_level)
    log_name=time.strftime("%Y-%m-%d", time.localtime()) + '_' + log_file
    log.log_to_file(log_path, log_name)
    

# def test_func():
#     log = Log()
#     log.log_to_console('INFO')
#     log.logger.info('one')
#     log.warn('two')
#     log.debug('three')
#     log.error('four')

# if __name__ == "__main__":
#     test_func()
 
