import os
import sys
from argparse import Action, ArgumentParser
from .execute import get_lpkg_path

class RemoveAction(Action):
    def __call__(self, parser, namespace, values, option_string = None):
        for app in values:
            if ':' not in app:
                print('Please input appname:appversion.')
                parser.exit()
        setattr(namespace, self.dest, values)

class ListAction(Action):
    def __call__(self, parser, namespace, values, option_string = None):
        if ':' not in values:
            print('Please input appname:appversion.')
            parser.exit()
        setattr(namespace, self.dest, values)

class Parser(object):
    """Argument parser that can handle arguments with our special
    placeholder.

    """

    def __init__(self):
        self._parser = ArgumentParser(epilog="Lmg Packager")
        self._add_arguments()

    def _add_arguments(self):
        """Adds arguments to parser."""
       # self._parser.add_argument(
       #         'LmgFile', 
       #         nargs = '*', 
       #         metavar= 'xxx.lmg', 
       #         default=[],
       #         help='the lmg file')
        self._parser.add_argument(
                '--instdir',
                metavar='path',  
                dest='path',
                default='/opt', 
                help='specify the installation directory')
        #self._parser.add_argument(
        #        '--gui', 
        #        action='store_true',
        #        default=False,
        #        help='GUI display information.')
        self._parser.add_argument(
                '-t', '--tempdir', 
                metavar = 'Unzip Dir', 
                dest = 'udir',
                default = os.path.join(get_lpkg_path(), 'tmp'),
                help = 'Specify the installation decompression directory')
        self._parser.add_argument(
                '-v', '--version', 
                action='version',
                version='%(prog)s version : 1.1.4', 
                help='show the version')
        self._parser.add_argument(
                '-d', '--debug', 
                action='store_true',
                default=False,
                help='turn on debugging')
        self._add_conflicting_arguments()

    def _add_conflicting_arguments(self):
        group = self._parser.add_mutually_exclusive_group(required = True)
        group.add_argument(
                '-i', '--install', 
                nargs='+', 
                metavar='xxx.lmg', 
                dest='lmgfile',
                default=[], 
                help='The location of the package to be installed')
        group.add_argument(
                '-r', '--remove', 
                nargs='+', 
                metavar='appname',
                dest='app', 
                default=[], 
                help='The name of the package to be uninstalled')
        group.add_argument(
                '-f', '--field',
                metavar='xxx.lmg',
                dest='lmgfield',
                help = 'Print package field contents to standard output')
        group.add_argument(
                '-l', '--list', 
                action='store_true',
                default=False,
                help='Show app list')
        group.add_argument(
                '-L', '--listfiles', 
                metavar='appname',
                dest='app_lf',
                help = 'List files installed to your system from package-name.')
#        group.add_argument(
#                '--init', 
#                action = 'store_true', 
#                default = False,
#                help = 'init the lpkg program')

    def _prepare_arguments(self, argv):
        """Prepares arguments by:

        - removing placeholder and moving arguments after it to beginning,
          we need this to distinguish arguments from `command` with ours;

        - adding `--` before `command`, so our parse would ignore arguments
          of `command`.

        """
        
        if argv and not argv[0].startswith('-') and argv[0] != '--':
            return ['--'] + argv
        else:
            return argv

    def parse(self, argv):
        arguments = self._prepare_arguments(argv[1:])
        return self._parser.parse_args(arguments)

    def print_usage(self):
        self._parser.print_usage(sys.stderr)

    def print_help(self):
        self._parser.print_help(sys.stderr)

